@@ -20,20 +20,22 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
 #include "gstspideridentity.h"
-
 #include "gstspider.h"
 
-GstElementDetails gst_spider_identity_details = {
+GST_DEBUG_CATEGORY_STATIC (gst_spider_identity_debug);
+#define GST_CAT_DEFAULT gst_spider_identity_debug
+
+static GstElementDetails gst_spider_identity_details = GST_ELEMENT_DETAILS (
   "SpiderIdentity",
   "Generic",
-  "LGPL",
   "Link between spider and outside elements",
-  VERSION,
-  "Benjamin Otte <in7y118@public.uni-hamburg.de>",
-  "(C) 2002",
-};
+  "Benjamin Otte <in7y118@public.uni-hamburg.de>"
+);
 
 
 /* generic templates 
@@ -42,14 +44,14 @@ GstElementDetails gst_spider_identity_details = {
 GST_PAD_TEMPLATE_FACTORY (spider_src_factory,
   "src",
   GST_PAD_SRC,
-  GST_PAD_REQUEST,
+  GST_PAD_ALWAYS,
   NULL      /* no caps */
 );
 
 GST_PAD_TEMPLATE_FACTORY (spider_sink_factory,
   "sink",
   GST_PAD_SINK,
-  GST_PAD_REQUEST,
+  GST_PAD_ALWAYS,
   NULL      /* no caps */
 );
 
@@ -103,7 +105,10 @@ gst_spider_identity_get_type (void)
       0,
       (GInstanceInitFunc)gst_spider_identity_init,
     };
-    spider_identity_type = g_type_register_static (GST_TYPE_ELEMENT, "GstSpiderIdentity", &spider_identity_info, 0);
+    spider_identity_type = g_type_register_static (GST_TYPE_ELEMENT, "GstSpiderIdentity", 
+						   &spider_identity_info, 0);
+    GST_DEBUG_CATEGORY_INIT (gst_spider_identity_debug, "spideridentity", 
+			     0, "spider autoplugging proxy element");
   }
   return spider_identity_type;
 }
@@ -118,6 +123,7 @@ gst_spider_identity_class_init (GstSpiderIdentityClass *klass)
   /* add our two pad templates */
   gst_element_class_add_pad_template (gstelement_class, GST_PAD_TEMPLATE_GET (spider_src_factory));
   gst_element_class_add_pad_template (gstelement_class, GST_PAD_TEMPLATE_GET (spider_sink_factory));
+  gst_element_class_set_details (gstelement_class, &gst_spider_identity_details);
   
   gstelement_class->change_state = GST_DEBUG_FUNCPTR(gst_spider_identity_change_state);
   gstelement_class->request_new_pad = GST_DEBUG_FUNCPTR(gst_spider_identity_request_new_pad);
@@ -151,7 +157,6 @@ gst_spider_identity_init (GstSpiderIdentity *ident)
 
   /* variables */
   ident->plugged = FALSE;
-  
 }
 
 static void 
@@ -178,19 +183,24 @@ gst_spider_identity_chain (GstPad *pad, GstBuffer *buf)
       {
 	GstSpiderConnection *conn = (GstSpiderConnection *) list->data;
 	list = g_list_next (list);
-	gst_element_set_eos (GST_ELEMENT (conn->src));
-        gst_pad_push (conn->src->src, GST_BUFFER (gst_event_new (GST_EVENT_EOS)));  
+        if (conn->current != (GstElement *) conn->src) {
+          GST_DEBUG ("sending EOS to unconnected element %s from %s", 
+	       GST_ELEMENT_NAME (conn->src), GST_ELEMENT_NAME (ident));
+          gst_pad_push (conn->src->src, GST_DATA (GST_BUFFER (gst_event_new (GST_EVENT_EOS))));  
+          gst_element_set_eos (GST_ELEMENT (conn->src));
+	}
       }
     }
     /* end hack for current event stuff here */
 
-    gst_pad_event_default (ident->sink, GST_EVENT (buf));
+    gst_pad_event_default (pad, GST_EVENT (buf));
     return;
   }
 
   if ((ident->src != NULL) && (GST_PAD_PEER (ident->src) != NULL)) {
     /* g_print("pushing buffer %p (refcount %d - buffersize %d) to pad %s:%s\n", buf, GST_BUFFER_REFCOUNT (buf), GST_BUFFER_SIZE (buf), GST_DEBUG_PAD_NAME (ident->src)); */
-    gst_pad_push (ident->src, buf);
+    GST_LOG ( "push %p %" G_GINT64_FORMAT, buf, GST_BUFFER_OFFSET (buf));
+    gst_pad_push (ident->src, GST_DATA (buf));
   } else if (GST_IS_BUFFER (buf)) {
     gst_buffer_unref (buf);
   }
@@ -198,8 +208,6 @@ gst_spider_identity_chain (GstPad *pad, GstBuffer *buf)
 GstSpiderIdentity*           
 gst_spider_identity_new_src (gchar *name)
 {
-  //GstSpiderIdentity *ret = (GstSpiderIdentity *) g_object_new (gst_spider_identity_get_type (), NULL);
-  //GST_ELEMENT_NAME (ret) = name;
   GstSpiderIdentity *ret = (GstSpiderIdentity *) gst_element_factory_make ("spideridentity", name);
   /* set the right functions */
   gst_element_set_loop_function (GST_ELEMENT (ret), (GstElementLoopFunction) GST_DEBUG_FUNCPTR (gst_spider_identity_src_loop));
@@ -209,9 +217,6 @@ gst_spider_identity_new_src (gchar *name)
 GstSpiderIdentity*           
 gst_spider_identity_new_sink (gchar *name)
 {
-  //GstSpiderIdentity *ret = (GstSpiderIdentity *) g_object_new (gst_spider_identity_get_type (), NULL);
-  
-  //GST_ELEMENT_NAME (ret) = name;
   GstSpiderIdentity *ret = (GstSpiderIdentity *) gst_element_factory_make ("spideridentity", name);
 
   /* set the right functions */
@@ -272,7 +277,7 @@ gst_spider_identity_request_new_pad  (GstElement *element, GstPadTemplate *templ
     case GST_PAD_SINK:
       if (ident->sink != NULL) break;
       /* sink */
-      GST_DEBUG(0, "element %s requests new sink pad", GST_ELEMENT_NAME(ident));
+      GST_DEBUG ( "element %s requests new sink pad", GST_ELEMENT_NAME(ident));
       ident->sink = gst_pad_new ("sink", GST_PAD_SINK);
       gst_element_add_pad (GST_ELEMENT (ident), ident->sink);
       gst_pad_set_link_function (ident->sink, GST_DEBUG_FUNCPTR (gst_spider_identity_link));
@@ -282,7 +287,7 @@ gst_spider_identity_request_new_pad  (GstElement *element, GstPadTemplate *templ
     case GST_PAD_SRC:
       /* src */
       if (ident->src != NULL) break;
-      GST_DEBUG(0, "element %s requests new src pad", GST_ELEMENT_NAME(ident));
+      GST_DEBUG ( "element %s requests new src pad", GST_ELEMENT_NAME(ident));
       ident->src = gst_pad_new ("src", GST_PAD_SRC);
       gst_element_add_pad (GST_ELEMENT (ident), ident->src);
       gst_pad_set_link_function (ident->src, GST_DEBUG_FUNCPTR (gst_spider_identity_link));
@@ -293,7 +298,7 @@ gst_spider_identity_request_new_pad  (GstElement *element, GstPadTemplate *templ
       break;
   }
   
-  GST_DEBUG(0, "element %s requested a new pad but none could be created", GST_ELEMENT_NAME(ident));
+  GST_DEBUG ( "element %s requested a new pad but none could be created", GST_ELEMENT_NAME(ident));
   return NULL;
 }
 
@@ -354,7 +359,7 @@ gst_spider_identity_start_type_finding (GstSpiderIdentity *ident)
   gchar *name;*/
   gboolean restart = FALSE;
   
-  GST_DEBUG (GST_CAT_AUTOPLUG, "element %s starts typefinding", GST_ELEMENT_NAME(ident));
+  GST_DEBUG ("element %s starts typefinding", GST_ELEMENT_NAME(ident));
   if (GST_STATE (GST_ELEMENT_PARENT (ident)) == GST_STATE_PLAYING)
   {
     gst_element_set_state (GST_ELEMENT (GST_ELEMENT_PARENT (ident)), GST_STATE_PAUSED);
@@ -381,7 +386,7 @@ gst_spider_identity_dumb_loop  (GstSpiderIdentity *ident)
   g_return_if_fail (GST_IS_SPIDER_IDENTITY (ident));
   g_assert (ident->sink != NULL);
 
-  buf = gst_pad_pull (ident->sink);
+  buf = GST_BUFFER (gst_pad_pull (ident->sink));
 
   gst_spider_identity_chain (ident->sink, buf);
 }
@@ -401,128 +406,109 @@ gst_spider_identity_src_loop (GstSpiderIdentity *ident)
     gst_spider_identity_dumb_loop (ident);
     return;
   }
-
   gst_element_interrupt (GST_ELEMENT (ident));
 }
 /* This loop function is only needed when typefinding.
  */
+typedef struct {
+  GstBuffer *buffer;
+  guint best_probability;
+  GstCaps *caps;
+} SpiderTypeFind;
+guint8 *
+spider_find_peek (gpointer data, gint64 offset, guint size)
+{
+  SpiderTypeFind *find = (SpiderTypeFind *) data;
+  gint64 buffer_offset = GST_BUFFER_OFFSET_IS_VALID (find->buffer) ? 
+			 GST_BUFFER_OFFSET (find->buffer) : 0;
+  
+  if (offset >= buffer_offset && offset + size <= buffer_offset + GST_BUFFER_SIZE (find->buffer)) {
+    GST_LOG ("peek %"G_GINT64_FORMAT", %u successful", offset, size);
+    return GST_BUFFER_DATA (find->buffer) + offset - buffer_offset;
+  } else {
+    GST_LOG ("peek %"G_GINT64_FORMAT", %u failed", offset, size);
+    return NULL;
+  }
+}
+static void
+spider_find_suggest (gpointer data, guint probability, GstCaps *caps)
+{
+  SpiderTypeFind *find = (SpiderTypeFind *) data;
+  G_GNUC_UNUSED gchar *caps_str;
+
+  caps_str = gst_caps_to_string (caps);
+  GST_INFO ("suggest %u, %s", probability, caps_str);
+  g_free (caps_str);
+  if (probability > find->best_probability) {
+    gst_caps_replace (&find->caps, caps);
+    find->best_probability = probability;
+  }
+}
 static void
 gst_spider_identity_sink_loop_type_finding (GstSpiderIdentity *ident)
 {
-  GstBuffer *buf=NULL;
-  GstBuffer *typefindbuf = NULL;
-  gboolean getmorebuf = TRUE;
-  GList *type_list;
-  gboolean restart_spider = FALSE;
-  GstCaps *caps;
+  GstData *data;
+  GstTypeFind gst_find;
+  SpiderTypeFind find;
+  GList *walk, *type_list = NULL;
 
-  /* this should possibly be a property */
-  /* this is so big because typefinding is bad.
-   * this is a cruel hack. We need another typefind system */
-  guint bufsizelimit = 150 * 1024;
-  
-  /* checks - disable for speed */
-  g_return_if_fail (ident != NULL);
   g_return_if_fail (GST_IS_SPIDER_IDENTITY (ident));
-  
-  while (getmorebuf){
 
-    /* check if our buffer is big enough to do a typefind */
-    if (typefindbuf && GST_BUFFER_SIZE(typefindbuf) >= bufsizelimit){
-      getmorebuf = FALSE;
-      break;
-    }
-    buf = gst_pad_pull (ident->sink);
-  
-    /* if it's an event... */
-    while (GST_IS_EVENT (buf)) {
-      switch (GST_EVENT_TYPE (GST_EVENT (buf))){
-      case GST_EVENT_EOS:
-        getmorebuf = FALSE;
-	/* FIXME Notify the srcs that EOS has happened */
-        gst_pad_event_default (ident->sink, GST_EVENT (buf));
-	break;
-      default:
-        gst_pad_event_default (ident->sink, GST_EVENT (buf));
-        buf = gst_pad_pull (ident->sink);
-        break;
-      }
-      /* handle DISCONT events, please */
-    }
-
-    if (!typefindbuf){
-      typefindbuf = buf;
-    }
-    else {
-      GstBuffer *oldbuf = typefindbuf;
-      typefindbuf = gst_buffer_merge(typefindbuf, buf);
-      gst_buffer_unref (oldbuf);
-      gst_buffer_unref (buf);
-    }
+  data = gst_pad_pull (ident->sink);
+  while (!GST_IS_BUFFER (data)) {
+    gst_spider_identity_chain (ident->sink, GST_BUFFER (data));
+    data = gst_pad_pull (ident->sink);
   }
   
-  if (!typefindbuf){
-    goto end;
-  }
-
+  find.buffer = GST_BUFFER (data);
   /* maybe there are already valid caps now? */
-  if ((caps = gst_pad_get_caps (ident->sink)) != NULL) {
+  if ((find.caps = gst_pad_get_caps (ident->sink)) != NULL) {
+    gst_caps_ref (find.caps); /* it's unrefed later below */
     goto plug;
   }
-
+  
   /* now do the actual typefinding with the supplied buffer */
-  type_list = (GList *) gst_type_get_list ();
+  walk = type_list = gst_type_find_factory_get_list ();
     
-  while (type_list) {
-    GstType *type = (GstType *) type_list->data;
-    GSList *factories = type->factories;
+  find.best_probability = 0;
+  find.caps = NULL;
+  gst_find.data = &find;
+  gst_find.peek = spider_find_peek;
+  gst_find.suggest = spider_find_suggest;
+  while (walk) {
+    GstTypeFindFactory *factory = GST_TYPE_FIND_FACTORY (walk->data);
 
-    while (factories) {
-      GstTypeFactory *factory = GST_TYPE_FACTORY (factories->data);
-      GstTypeFindFunc typefindfunc = (GstTypeFindFunc)factory->typefindfunc;
-
-      if (typefindfunc && (caps = typefindfunc (typefindbuf, factory))) {
-
-	if (gst_pad_try_set_caps (ident->sink, caps) <= 0) {
-          g_warning ("typefind: found type but peer didn't accept it");
-          gst_caps_sink (caps);
-	} else {
-          goto plug;
-        }
-      }
-      factories = g_slist_next (factories);
-    }
-    type_list = g_list_next (type_list);
+    GST_DEBUG ("trying typefind function %s", GST_PLUGIN_FEATURE_NAME (factory));
+    gst_type_find_factory_call_function (factory, &gst_find);
+    if (find.best_probability >= GST_TYPE_FIND_MAXIMUM)
+      goto plug;
+    walk = g_list_next (walk);
   }
+  if (find.best_probability > 0)
+    goto plug;
   gst_element_error(GST_ELEMENT(ident), "Could not find media type", NULL);
-  gst_buffer_unref(typefindbuf);
-  typefindbuf = GST_BUFFER (gst_event_new (GST_EVENT_EOS));
+  find.buffer = GST_BUFFER (gst_event_new (GST_EVENT_EOS));
 
 end:
-
   /* remove loop function */
   gst_element_set_loop_function (GST_ELEMENT (ident), 
                                 (GstElementLoopFunction) GST_DEBUG_FUNCPTR (gst_spider_identity_dumb_loop));
   
   /* push the buffer */
-  gst_spider_identity_chain (ident->sink, typefindbuf);
-
+  gst_spider_identity_chain (ident->sink, find.buffer);
+  
   return;
 
 plug:
-  gst_caps_debug (caps, "spider starting caps");
-  gst_caps_sink (caps);
-  /* pause the autoplugger */
-  if (gst_element_get_state (GST_ELEMENT (GST_ELEMENT_PARENT(ident))) == GST_STATE_PLAYING) {
-    gst_element_set_state (GST_ELEMENT (GST_ELEMENT_PARENT(ident)), GST_STATE_PAUSED);
-    restart_spider = TRUE;
-  }
-  gst_spider_identity_plug (ident);  
+  GST_INFO ("typefind function found caps"); 
+  g_assert (gst_pad_try_set_caps (ident->src, find.caps) > 0);
+  gst_caps_debug (find.caps, "spider starting caps");
+  gst_caps_unref (find.caps);
+  if (type_list)
+    g_list_free (type_list);
+
+  gst_spider_identity_plug (ident);
 
-  /* restart autoplugger */
-  if (restart_spider){
-    gst_element_set_state (GST_ELEMENT (GST_ELEMENT_PARENT(ident)), GST_STATE_PLAYING);
-  }
   goto end;
 }
 
@@ -532,7 +518,7 @@ gst_spider_identity_handle_src_event (GstPad *pad, GstEvent *event)
   gboolean res = TRUE;
   GstSpiderIdentity *ident;
 
-  GST_DEBUG (0, "spider_identity src_event\n");
+  GST_DEBUG ( "spider_identity src_event");
 
   ident = GST_SPIDER_IDENTITY (gst_pad_get_parent (pad));
 
@@ -546,5 +532,3 @@ gst_spider_identity_handle_src_event (GstPad *pad, GstEvent *event)
 
   return res;
 }
-
-
