@@ -31,15 +31,12 @@
 #include "gstscheduler.h"
 #include "gstindex.h"
 
-GstElementDetails gst_bin_details = {
+static GstElementDetails gst_bin_details = GST_ELEMENT_DETAILS (
   "Generic bin",
   "Generic/Bin",
-  "LGPL",
   "Simple container object",
-  VERSION,
-  "Erik Walthinsen <omega@cse.ogi.edu>",
-  "(C) 1999",
-};
+  "Erik Walthinsen <omega@cse.ogi.edu>"
+);
 
 GType _gst_bin_type = 0;
 
@@ -85,6 +82,7 @@ enum
   /* FILL ME */
 };
 
+static void 			gst_bin_base_init 		(gpointer g_class);
 static void 			gst_bin_class_init 		(GstBinClass * klass);
 static void 			gst_bin_init 			(GstBin * bin);
 
@@ -97,7 +95,7 @@ gst_bin_get_type (void)
   if (!_gst_bin_type) {
     static const GTypeInfo bin_info = {
       sizeof (GstBinClass),
-      NULL,
+      gst_bin_base_init,
       NULL,
       (GClassInitFunc) gst_bin_class_init,
       NULL,
@@ -113,6 +111,14 @@ gst_bin_get_type (void)
   return _gst_bin_type;
 }
 
+static void
+gst_bin_base_init (gpointer g_class)
+{
+  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
+  
+  gst_element_class_set_details (gstelement_class, &gst_bin_details);
+}
+
 static void
 gst_bin_class_init (GstBinClass * klass)
 {
@@ -129,11 +135,11 @@ gst_bin_class_init (GstBinClass * klass)
   gst_bin_signals[ELEMENT_ADDED] =
     g_signal_new ("element_added", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GstBinClass, element_added), NULL, NULL,
-		  gst_marshal_VOID__POINTER, G_TYPE_NONE, 1, G_TYPE_POINTER);
+		  gst_marshal_VOID__POINTER, G_TYPE_NONE, 1, GST_TYPE_ELEMENT);
   gst_bin_signals[ELEMENT_REMOVED] =
     g_signal_new ("element_removed", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GstBinClass, element_removed), NULL, NULL,
-		  gst_marshal_VOID__POINTER, G_TYPE_NONE, 1, G_TYPE_POINTER);
+		  gst_marshal_VOID__POINTER, G_TYPE_NONE, 1, GST_TYPE_ELEMENT);
   gst_bin_signals[ITERATE] =
     g_signal_new ("iterate", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GstBinClass, iterate),
