@@ -30,49 +30,50 @@ G_BEGIN_DECLS
 
 typedef struct _GstCaps GstCaps;
 
+#define	GST_CAPS_TRACE_NAME "GstCaps"
+
 extern GType _gst_caps_type;
 
 #define GST_TYPE_CAPS  (_gst_caps_type)
 
 typedef enum {
-  GST_CAPS_FIXED        = (1 << 0),     /* caps has no variable properties */
-  GST_CAPS_FLOATING     = (1 << 1)      /* caps is floating */
+  GST_CAPS_UNUSED 	= (1 << 0),	/* unused flag */
+  GST_CAPS_FLOATING 	= (1 << 1)	/* caps is floating */
 } GstCapsFlags;
 
-#define GST_CAPS(caps)          ((GstCaps *)(caps))
+#define GST_CAPS(caps)  	((GstCaps *)(caps))
 
-#define GST_CAPS_FLAGS(caps)            ((caps)->flags)
-#define GST_CAPS_FLAG_IS_SET(caps,flag) (GST_CAPS_FLAGS (caps) & flag)
-#define GST_CAPS_FLAG_SET(caps,flag)    (GST_CAPS_FLAGS (caps) |= (flag))
-#define GST_CAPS_FLAG_UNSET(caps,flag)  (GST_CAPS_FLAGS (caps) &= ~(flag))
+#define GST_CAPS_FLAGS(caps)		((caps)->flags)
+#define GST_CAPS_FLAG_IS_SET(caps,flag)	(GST_CAPS_FLAGS (caps) &   (flag))
+#define GST_CAPS_FLAG_SET(caps,flag)	(GST_CAPS_FLAGS (caps) |=  (flag))
+#define GST_CAPS_FLAG_UNSET(caps,flag)	(GST_CAPS_FLAGS (caps) &= ~(flag))
 
-#define GST_CAPS_REFCOUNT(caps)         ((caps)->refcount)
-#define GST_CAPS_PROPERTIES(caps)       ((caps)->properties)
-#define GST_CAPS_NEXT(caps)             ((caps)->next)
+#define GST_CAPS_REFCOUNT(caps)		((caps)->refcount)
+#define GST_CAPS_PROPERTIES(caps)  	((caps)->properties)
+#define GST_CAPS_NEXT(caps)  		((caps)->next)
 
-/* GST_CAPS_IS_FIXED is wrong, please use the backported function */
-#define GST_CAPS_IS_FIXED(caps)         (GST_CAPS_FLAGS (caps) & GST_CAPS_FIXED)
-#define GST_CAPS_IS_FIXED_BP(caps)      (((caps)->properties == NULL) || \
-                                         (GST_PROPS_IS_FIXED ((caps)->properties)))
-#define GST_CAPS_IS_FLOATING(caps)      (GST_CAPS_FLAGS (caps) & GST_CAPS_FLOATING)
+#define GST_CAPS_IS_FIXED(caps)		(((caps)->properties == NULL) || \
+					 (GST_PROPS_IS_FIXED ((caps)->properties)))
+#define GST_CAPS_IS_FLOATING(caps)	(GST_CAPS_FLAG_IS_SET ((caps), GST_CAPS_FLOATING))
+#define GST_CAPS_IS_CHAINED(caps)  	(GST_CAPS_NEXT (caps) != NULL)
 
-#define GST_CAPS_IS_CHAINED(caps)       (GST_CAPS_NEXT (caps) != NULL)
+#define GST_CAPS_NONE			NULL
+#define GST_CAPS_ANY			(gst_caps_get_any())
 
 struct _GstCaps {
-  /* --- public ReadOnly --- */
-  gchar         *name;                  /* the name of this caps */
-  guint16        id;                    /* type id (major type) representing 
-                                           the mime type, it's stored as a GQuark 
-                                           for speed/space reasons */
+  /* --- public --- */
+  gchar 	*name;			/* the name of this caps */
+  GQuark 	 id;			/* type id (major type) representing 
+					   the mime type, it's stored as a GQuark 
+					   for speed/space reasons */
 
-  guint16        flags;                 /* flags */
-  guint          refcount;
+  guint16 	 flags;			/* flags */
+  guint 	 refcount;		
 
-  GstProps      *properties;            /* properties for this capability */
-  GstCaps       *next;                  /* not with a GList for efficiency */
+  GstProps 	*properties;		/* properties for this capability */
+  GstCaps 	*next;			/* not with a GList for efficiency */
 };
 
-
 /* factory macros which make it easier for plugins to instantiate */
 
 #ifdef G_HAVE_ISO_VARARGS
@@ -92,7 +93,7 @@ factoryname (void)                              \
   if (!caps) {                              	\
     caps = gst_caps_chain (__VA_ARGS__, NULL); 	\
   }                                             \
-  return caps;                              	\
+  return gst_caps_ref(caps);                   	\
 }
 #elif defined(G_HAVE_GNUC_VARARGS)
 #define GST_CAPS_NEW(name, type, a...)          \
@@ -111,31 +112,35 @@ factoryname (void)                              \
   if (!caps) {                              	\
     caps = gst_caps_chain (a, NULL);      	\
   }                                             \
-  return caps;                              	\
+  return gst_caps_ref(caps);                   	\
 }
 #endif
 
+/* get caps from a factory */
 #define GST_CAPS_GET(fact) (fact)()
 
 
 /* initialize the subsystem */
 void		_gst_caps_initialize			(void);
 
+/* creating new caps */
+GType		gst_caps_get_type			(void);
 GstCaps*	gst_caps_new				(const gchar *name, const gchar *mime, GstProps *props);
-GstCaps*	gst_caps_new_id				(const gchar *name, const guint16 id, GstProps *props);
-
+GstCaps*	gst_caps_new_id				(const gchar *name, const GQuark id, GstProps *props);
+GstCaps*	gst_caps_get_any			(void);
 /* replace pointer to caps, doing proper refcounting */
-void            gst_caps_replace                        (GstCaps **oldcaps, GstCaps *newcaps);
-void            gst_caps_replace_sink                   (GstCaps **oldcaps, GstCaps *newcaps);
+void		gst_caps_replace			(GstCaps **oldcaps, GstCaps *newcaps);
+void		gst_caps_replace_sink			(GstCaps **oldcaps, GstCaps *newcaps);
 
 /* caps lifecycle control */
 GstCaps*	gst_caps_unref				(GstCaps *caps);
 GstCaps*	gst_caps_ref				(GstCaps *caps);
-void            gst_caps_sink                           (GstCaps *caps);
-void		gst_caps_destroy			(GstCaps *caps);
+void		gst_caps_sink				(GstCaps *caps);
 
+/* write debug lines to the log */
 void		gst_caps_debug				(GstCaps *caps, const gchar *label);
 
+/* copy caps */
 GstCaps*	gst_caps_copy				(GstCaps *caps);
 GstCaps*	gst_caps_copy_1				(GstCaps *caps);
 GstCaps*	gst_caps_copy_on_write			(GstCaps *caps);
@@ -146,9 +151,6 @@ void		gst_caps_set_name			(GstCaps *caps, const gchar *name);
 const gchar*	gst_caps_get_mime			(GstCaps *caps);
 void		gst_caps_set_mime			(GstCaps *caps, const gchar *mime);
 
-guint16		gst_caps_get_type_id			(GstCaps *caps);
-void		gst_caps_set_type_id			(GstCaps *caps, guint16 type_id);
-
 GstCaps*	gst_caps_set_props			(GstCaps *caps, GstProps *props);
 GstProps*	gst_caps_get_props			(GstCaps *caps);
 
@@ -166,19 +168,24 @@ GstProps*	gst_caps_get_props			(GstCaps *caps);
 #define		gst_caps_get_boolean(caps,name,res)	gst_props_entry_get_boolean(gst_props_get_entry((caps)->properties,name),res)
 #define		gst_caps_get_string(caps,name,res)	gst_props_entry_get_string(gst_props_get_entry((caps)->properties,name),res)
 
-#define		gst_caps_has_property(caps, name)	gst_props_has_property ((caps)->properties, name)
-#define		gst_caps_has_property_typed(caps, name, type)	gst_props_has_property_typed ((caps)->properties, name, type)
-#define		gst_caps_has_fixed_property(caps, name)	gst_props_has_fixed_property ((caps)->properties, name)
+gboolean	gst_caps_has_property			(GstCaps *caps, const gchar *name);
+gboolean	gst_caps_has_property_typed		(GstCaps *caps, const gchar *name, GstPropsType type);
+gboolean	gst_caps_has_fixed_property		(GstCaps *caps, const gchar *name);
 
 GstCaps*	gst_caps_get_by_name			(GstCaps *caps, const gchar *name);
 
-GstCaps* 	gst_caps_next 				(GstCaps *caps);
+/* use and construct chained caps */
+GstCaps*	gst_caps_next				(GstCaps *caps); 
 GstCaps*	gst_caps_chain				(GstCaps *caps, ...); 
 GstCaps*	gst_caps_append				(GstCaps *caps, GstCaps *capstoadd); 
 GstCaps*	gst_caps_prepend			(GstCaps *caps, GstCaps *capstoadd); 
 
+/* see if fromcaps is a subset of tocaps */
 gboolean	gst_caps_is_always_compatible		(GstCaps *fromcaps, GstCaps *tocaps);
+
+/* operations on caps */
 GstCaps*	gst_caps_intersect			(GstCaps *caps1, GstCaps *caps2);
+GstCaps*	gst_caps_union				(GstCaps *caps1, GstCaps *caps2);
 GstCaps*	gst_caps_normalize			(GstCaps *caps);
 
 #ifndef GST_DISABLE_LOADSAVE
@@ -186,6 +193,10 @@ xmlNodePtr      gst_caps_save_thyself			(GstCaps *caps, xmlNodePtr parent);
 GstCaps*	gst_caps_load_thyself			(xmlNodePtr parent);
 #endif
 
+/* for debugging purposes */
+gchar *		gst_caps_to_string			(GstCaps *caps);
+GstCaps *	gst_caps_from_string			(gchar *str);
+
 G_END_DECLS
 
 #endif /* __GST_CAPS_H__ */
