@@ -22,24 +22,32 @@
 
 #include <sys/time.h>
 
-/* #define GST_DEBUG_ENABLED */
 #include "gst_private.h"
 
 #include "gstclock.h"
-#include "gstlog.h"
+#include "gstinfo.h"
 #include "gstmemchunk.h"
 
+#ifndef GST_DISABLE_TRACE
+/* #define GST_WITH_ALLOC_TRACE */
+#include "gsttrace.h"
+static GstAllocTrace *_gst_clock_entry_trace;
+#endif
+
+#define DEFAULT_MAX_DIFF	(2 * GST_SECOND)
+
 enum {
   ARG_0,
   ARG_STATS,
+  ARG_MAX_DIFF
 };
 
-#define CLASS(clock)  GST_CLOCK_CLASS (G_OBJECT_GET_CLASS (clock))
-
-static GstMemChunk *_gst_clock_entries_chunk;
+static GstMemChunk   *_gst_clock_entries_chunk;
 
 static void		gst_clock_class_init		(GstClockClass *klass);
 static void		gst_clock_init			(GstClock *clock);
+static void 		gst_clock_dispose 		(GObject *object);
+
 static void             gst_clock_set_property		(GObject *object, guint prop_id, 
 							 const GValue *value, GParamSpec *pspec);
 static void             gst_clock_get_property		(GObject *object, guint prop_id, 
@@ -50,16 +58,16 @@ static void		gst_clock_update_stats		(GstClock *clock);
 static GstObjectClass *parent_class = NULL;
 /* static guint gst_clock_signals[LAST_SIGNAL] = { 0 }; */
 
-static GMutex *_gst_clock_mutex;
-static GCond  *_gst_clock_cond;
-
-static inline GstClockID
+static GstClockID
 gst_clock_entry_new (GstClock *clock, GstClockTime time, 
 		     GstClockTime interval, GstClockEntryType type)
 {
   GstClockEntry *entry;
 
   entry = gst_mem_chunk_alloc (_gst_clock_entries_chunk);
+#ifndef GST_DISABLE_TRACE
+  gst_alloc_trace_new (_gst_clock_entry_trace, entry);
+#endif
 
   entry->clock = clock;
   entry->time = time;
@@ -92,7 +100,7 @@ gst_clock_new_single_shot_id (GstClock *clock, GstClockTime time)
 }
 
 /**
- * gst_clock_new_periodic__id
+ * gst_clock_new_periodic_id
  * @clock: The clockid to get a periodic notification id from
  * @start_time: the requested start time
  * @interval: the requested interval
@@ -108,7 +116,7 @@ gst_clock_new_periodic_id (GstClock *clock, GstClockTime start_time,
                            GstClockTime interval)
 {
   g_return_val_if_fail (GST_IS_CLOCK (clock), NULL);
-  g_return_val_if_fail (start_time != GST_CLOCK_TIME_NONE, NULL);
+  g_return_val_if_fail (GST_CLOCK_TIME_IS_VALID (start_time), NULL);
   g_return_val_if_fail (interval != 0, NULL);
 
   return gst_clock_entry_new (clock, 
@@ -151,26 +159,38 @@ gst_clock_id_wait (GstClockID id, GstClockTimeDiff *jitter)
   GstClock *clock;
   GstClockReturn res = GST_CLOCK_UNSUPPORTED;
   GstClockTime requested;
+  GstClockClass *cclass;
   
   g_return_val_if_fail (id != NULL, GST_CLOCK_ERROR);
 
   entry = (GstClockEntry *) id;
   requested = GST_CLOCK_ENTRY_TIME (entry);
 
-  if (requested == GST_CLOCK_TIME_NONE) {
-    res = GST_CLOCK_TIMEOUT;
-    goto done;
+  if (! GST_CLOCK_TIME_IS_VALID (requested)) {
+    GST_CAT_DEBUG (GST_CAT_CLOCK, "invalid time requested, returning _TIMEOUT");
+    return GST_CLOCK_TIMEOUT;
   }
-  
-  clock = GST_CLOCK_ENTRY_CLOCK (entry);
-  
-  if (CLASS (clock)->wait) {
-    GstClockTime now;
 
+  clock = GST_CLOCK_ENTRY_CLOCK (entry);
+  cclass = GST_CLOCK_GET_CLASS (clock);
+  
+  if (cclass->wait) {
+    GstClockTime now;
+    
+    GST_LOCK (clock);
+    clock->entries = g_list_prepend (clock->entries, entry);
+    GST_UNLOCK (clock);
+
+    GST_CAT_DEBUG (GST_CAT_CLOCK, "waiting on clock");
     do {
-      res = CLASS (clock)->wait (clock, entry);
+      res = cclass->wait (clock, entry);
     }
     while (res == GST_CLOCK_ENTRY_RESTART);
+    GST_CAT_DEBUG (GST_CAT_CLOCK, "done waiting");
+
+    GST_LOCK (clock);
+    clock->entries = g_list_remove (clock->entries, entry);
+    GST_UNLOCK (clock);
 
     if (jitter) {
       now = gst_clock_get_time (clock);
@@ -182,24 +202,17 @@ gst_clock_id_wait (GstClockID id, GstClockTimeDiff *jitter)
     }
   }
 
-done:
-  if (entry->type == GST_CLOCK_ENTRY_SINGLE) {
-    gst_clock_id_free (id);
-  }
-
   return res;
 }
 
 /**
- * gst_clock_wait_async
- * @clock: a #GstClock to wait on
- * @time: The #GstClockTime to wait for
+ * gst_clock_id_wait_async:
+ * @id: a #GstClockID to wait on
  * @func: The callback function 
  * @user_data: User data passed in the calback
  *
- * Register a callback on the given clock that will be triggered 
- * when the clock has reached the given time. A ClockID is returned
- * that can be used to cancel the request.
+ * Register a callback on the given clockid with the given
+ * function and user_data.
  *
  * Returns: the result of the non blocking wait.
  */
@@ -210,6 +223,7 @@ gst_clock_id_wait_async (GstClockID id,
   GstClockEntry *entry;
   GstClock *clock;
   GstClockReturn res = GST_CLOCK_UNSUPPORTED;
+  GstClockClass *cclass;
   
   g_return_val_if_fail (id != NULL, GST_CLOCK_ERROR);
   g_return_val_if_fail (func != NULL, GST_CLOCK_ERROR);
@@ -217,44 +231,57 @@ gst_clock_id_wait_async (GstClockID id,
   entry = (GstClockEntry *) id;
   clock = entry->clock;
 
-  if (GST_CLOCK_ENTRY_TIME (entry) == GST_CLOCK_TIME_NONE) {
+  if (! GST_CLOCK_TIME_IS_VALID (GST_CLOCK_ENTRY_TIME (entry))) {
     (func) (clock, GST_CLOCK_TIME_NONE, id, user_data);
     return GST_CLOCK_TIMEOUT;
   }
 
-  if (CLASS (clock)->wait_async) {
-    res = CLASS (clock)->wait_async (clock, entry, func, user_data);
+  cclass = GST_CLOCK_GET_CLASS (clock);
+
+  if (cclass->wait_async) {
+    entry->func = func;
+    entry->user_data = user_data;
+
+    res = cclass->wait_async (clock, entry);
   }
 
   return res;
 }
 
+static void
+gst_clock_reschedule_func (GstClockEntry *entry)
+{
+  entry->status = GST_CLOCK_ENTRY_OK;
+  
+  gst_clock_id_unlock ((GstClockID)entry);
+}
+
 /**
- * gst_clock_remove_id
- * @clock: The clock to cancel the request on
- * @id: The id to cancel
+ * gst_clock_id_unschedule:
+ * @id: The id to unschedule
  *
  * Cancel an outstanding async notification request with the given ID.
- * This can be an ID generated with gst_clock_wait_async() or 
- * gst_clock_notify_async().
  */
 void
 gst_clock_id_unschedule (GstClockID id)
 {
   GstClockEntry *entry;
   GstClock *clock;
+  GstClockClass *cclass;
   
   g_return_if_fail (id != NULL);
 
   entry = (GstClockEntry *) id;
   clock = entry->clock;
 
-  if (CLASS (clock)->unschedule)
-    CLASS (clock)->unschedule (clock, entry);
+  cclass = GST_CLOCK_GET_CLASS (clock);
+
+  if (cclass->unschedule)
+    cclass->unschedule (clock, entry);
 }
 
 /**
- * gst_clock_id_free
+ * gst_clock_id_free:
  * @id: The clockid to free
  *
  * Free the resources held by the given id
@@ -264,11 +291,14 @@ gst_clock_id_free (GstClockID id)
 {
   g_return_if_fail (id != NULL);
 
+#ifndef GST_DISABLE_TRACE
+  gst_alloc_trace_free (_gst_clock_entry_trace, id);
+#endif
   gst_mem_chunk_free (_gst_clock_entries_chunk, id);
 }
 
 /**
- * gst_clock_unlock_id
+ * gst_clock_id_unlock:
  * @id: The clockid to unlock
  *
  * Unlock the givan ClockID.
@@ -278,14 +308,17 @@ gst_clock_id_unlock (GstClockID id)
 {
   GstClockEntry *entry;
   GstClock *clock;
+  GstClockClass *cclass;
   
   g_return_if_fail (id != NULL);
 
   entry = (GstClockEntry *) id;
   clock = entry->clock;
 
-  if (CLASS (clock)->unlock)
-    CLASS (clock)->unlock (clock, entry);
+  cclass = GST_CLOCK_GET_CLASS (clock);
+
+  if (cclass->unlock)
+    cclass->unlock (clock, entry);
 }
 
 
@@ -334,20 +367,27 @@ gst_clock_class_init (GstClockClass *klass)
                      sizeof (GstClockEntry), sizeof (GstClockEntry) * 32,
                      G_ALLOC_AND_FREE);
 
-  _gst_clock_mutex = g_mutex_new ();
-  _gst_clock_cond  = g_cond_new ();
+#ifndef GST_DISABLE_TRACE
+  _gst_clock_entry_trace = gst_alloc_trace_register (GST_CLOCK_ENTRY_TRACE_NAME);
+#endif
 
+  gobject_class->dispose      = GST_DEBUG_FUNCPTR (gst_clock_dispose);
   gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_clock_set_property);
   gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_clock_get_property);
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_STATS,
     g_param_spec_boolean ("stats", "Stats", "Enable clock stats",
                           FALSE, G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MAX_DIFF,
+    g_param_spec_int64 ("max-diff", "Max diff", "The maximum amount of time to wait in nanoseconds",
+                        0, G_MAXINT64, DEFAULT_MAX_DIFF, G_PARAM_READWRITE));
 }
 
 static void
 gst_clock_init (GstClock *clock)
 {
+  clock->max_diff = DEFAULT_MAX_DIFF;
+
   clock->speed = 1.0;
   clock->active = FALSE;
   clock->start_time = 0;
@@ -360,6 +400,17 @@ gst_clock_init (GstClock *clock)
   clock->active_cond = g_cond_new ();
 }
 
+static void
+gst_clock_dispose (GObject *object)
+{
+  GstClock *clock = GST_CLOCK (object);
+
+  g_mutex_free (clock->active_mutex);
+  g_cond_free (clock->active_cond);
+
+  G_OBJECT_CLASS (parent_class)->dispose (object);
+}
+
 /**
  * gst_clock_set_speed
  * @clock: a #GstClock to modify
@@ -373,10 +424,14 @@ gst_clock_init (GstClock *clock)
 gdouble
 gst_clock_set_speed (GstClock *clock, gdouble speed)
 {
+  GstClockClass *cclass;
+
   g_return_val_if_fail (GST_IS_CLOCK (clock), 0.0);
 
-  if (CLASS (clock)->change_speed)
-    clock->speed = CLASS (clock)->change_speed (clock, clock->speed, speed);
+  cclass = GST_CLOCK_GET_CLASS (clock);
+
+  if (cclass->change_speed)
+    clock->speed = cclass->change_speed (clock, clock->speed, speed);
 
   return clock->speed;
 }
@@ -409,11 +464,15 @@ gst_clock_get_speed (GstClock *clock)
 guint64
 gst_clock_set_resolution (GstClock *clock, guint64 resolution)
 {
-  g_return_val_if_fail (GST_IS_CLOCK (clock), 0LL);
-  g_return_val_if_fail (resolution != 0, 0LL);
+  GstClockClass *cclass;
 
-  if (CLASS (clock)->change_resolution)
-    clock->resolution = CLASS (clock)->change_resolution (clock, clock->resolution, resolution);
+  g_return_val_if_fail (GST_IS_CLOCK (clock), G_GINT64_CONSTANT (0));
+  g_return_val_if_fail (resolution != 0, G_GINT64_CONSTANT (0));
+
+  cclass = GST_CLOCK_GET_CLASS (clock);
+
+  if (cclass->change_resolution)
+    clock->resolution = cclass->change_resolution (clock, clock->resolution, resolution);
 
   return clock->resolution;
 }
@@ -429,12 +488,16 @@ gst_clock_set_resolution (GstClock *clock, guint64 resolution)
 guint64
 gst_clock_get_resolution (GstClock *clock)
 {
-  g_return_val_if_fail (GST_IS_CLOCK (clock), 0LL);
+  GstClockClass *cclass;
 
-  if (CLASS (clock)->get_resolution)
-    return CLASS (clock)->get_resolution (clock);
+  g_return_val_if_fail (GST_IS_CLOCK (clock), G_GINT64_CONSTANT (0));
 
-  return 1LL;
+  cclass = GST_CLOCK_GET_CLASS (clock);
+
+  if (cclass->get_resolution)
+    return cclass->get_resolution (clock);
+
+  return G_GINT64_CONSTANT (1);
 }
 
 /**
@@ -448,14 +511,17 @@ gst_clock_get_resolution (GstClock *clock)
 void
 gst_clock_set_active (GstClock *clock, gboolean active)
 {
-  GstClockTime time = 0LL;
+  GstClockTime time = G_GINT64_CONSTANT (0);
+  GstClockClass *cclass;
 
   g_return_if_fail (GST_IS_CLOCK (clock));
 
   clock->active = active;
+
+  cclass = GST_CLOCK_GET_CLASS (clock);
 	        
-  if (CLASS (clock)->get_internal_time) {
-    time = CLASS (clock)->get_internal_time (clock);
+  if (cclass->get_internal_time) {
+    time = cclass->get_internal_time (clock);
   }
 
   GST_LOCK (clock);
@@ -467,6 +533,7 @@ gst_clock_set_active (GstClock *clock, gboolean active)
     clock->last_time = time - clock->start_time;
     clock->accept_discont = FALSE;
   }
+  g_list_foreach (clock->entries, (GFunc) gst_clock_reschedule_func, NULL);
   GST_UNLOCK (clock);
 
   g_mutex_lock (clock->active_mutex);	
@@ -499,18 +566,22 @@ gst_clock_is_active (GstClock *clock)
 void
 gst_clock_reset (GstClock *clock)
 {
-  GstClockTime time = 0LL;
+  GstClockTime time = G_GINT64_CONSTANT (0);
+  GstClockClass *cclass;
 
   g_return_if_fail (GST_IS_CLOCK (clock));
 
-  if (CLASS (clock)->get_internal_time) {
-    time = CLASS (clock)->get_internal_time (clock);
+  cclass = GST_CLOCK_GET_CLASS (clock);
+	        
+  if (cclass->get_internal_time) {
+    time = cclass->get_internal_time (clock);
   }
 
   GST_LOCK (clock);
   clock->active = FALSE;
   clock->start_time = time;
-  clock->last_time = 0LL;
+  clock->last_time = G_GINT64_CONSTANT (0);
+  g_list_foreach (clock->entries, (GFunc) gst_clock_reschedule_func, NULL);
   GST_UNLOCK (clock);
 }
 
@@ -528,24 +599,28 @@ gst_clock_reset (GstClock *clock)
 gboolean
 gst_clock_handle_discont (GstClock *clock, guint64 time)
 {
-  GstClockTime itime = 0LL;
+  GstClockTime itime = G_GINT64_CONSTANT (0);
 
-  GST_DEBUG (GST_CAT_CLOCK, "clock discont %" G_GUINT64_FORMAT
+  GST_CAT_DEBUG (GST_CAT_CLOCK, "clock discont %" G_GUINT64_FORMAT
 		            " %" G_GUINT64_FORMAT " %d",
 			    time, clock->start_time, clock->accept_discont);
 
-  if (time == GST_CLOCK_TIME_NONE)
+  if (! GST_CLOCK_TIME_IS_VALID (time))
     return TRUE;
 
   GST_LOCK (clock);
   if (clock->accept_discont) {
-    if (CLASS (clock)->get_internal_time) {
-      itime = CLASS (clock)->get_internal_time (clock);
+    GstClockClass *cclass;
+
+    cclass = GST_CLOCK_GET_CLASS (clock);
+	  
+    if (cclass->get_internal_time) {
+      itime = cclass->get_internal_time (clock);
     }
   }
   else {
     GST_UNLOCK (clock);
-    GST_DEBUG (GST_CAT_CLOCK, "clock discont refused %" G_GUINT64_FORMAT
+    GST_CAT_DEBUG (GST_CAT_CLOCK, "clock discont refused %" G_GUINT64_FORMAT
 		              " %" G_GUINT64_FORMAT,
 			      time, clock->start_time);
     return FALSE;
@@ -554,9 +629,10 @@ gst_clock_handle_discont (GstClock *clock, guint64 time)
   clock->start_time = itime - time;
   clock->last_time = time;
   clock->accept_discont = FALSE;
+  g_list_foreach (clock->entries, (GFunc) gst_clock_reschedule_func, NULL);
   GST_UNLOCK (clock);
 
-  GST_DEBUG (GST_CAT_CLOCK, "new time %" G_GUINT64_FORMAT,
+  GST_CAT_DEBUG (GST_CAT_CLOCK, "new time %" G_GUINT64_FORMAT,
 	     gst_clock_get_time (clock));
 
   g_mutex_lock (clock->active_mutex);
@@ -578,17 +654,21 @@ gst_clock_handle_discont (GstClock *clock, guint64 time)
 GstClockTime
 gst_clock_get_time (GstClock *clock)
 {
-  GstClockTime ret = 0LL;
+  GstClockTime ret = G_GINT64_CONSTANT (0);
 
-  g_return_val_if_fail (GST_IS_CLOCK (clock), 0LL);
+  g_return_val_if_fail (GST_IS_CLOCK (clock), G_GINT64_CONSTANT (0));
 
   if (!clock->active) {
-    /* clock is not activen return previous time */
+    /* clock is not active return previous time */
     ret = clock->last_time;
   }
   else {
-    if (CLASS (clock)->get_internal_time) {
-      ret = CLASS (clock)->get_internal_time (clock) - clock->start_time;
+    GstClockClass *cclass;
+
+    cclass = GST_CLOCK_GET_CLASS (clock);
+
+    if (cclass->get_internal_time) {
+      ret = cclass->get_internal_time (clock) - clock->start_time;
     }
     /* make sure the time is increasing, else return last_time */
     if ((gint64) ret < (gint64) clock->last_time) {
@@ -642,6 +722,9 @@ gst_clock_set_property (GObject *object, guint prop_id,
     case ARG_STATS:
       clock->stats = g_value_get_boolean (value);
       break;
+    case ARG_MAX_DIFF:
+      clock->max_diff = g_value_get_int64 (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -660,6 +743,9 @@ gst_clock_get_property (GObject *object, guint prop_id,
     case ARG_STATS:
       g_value_set_boolean (value, clock->stats);
       break;
+    case ARG_MAX_DIFF:
+      g_value_set_int64 (value, clock->max_diff);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
