@@ -21,58 +21,6 @@ gst_object_flags_get_type (void)
 }
 
 
-/* enumerations from "gsttypes.h" */
-GType
-gst_element_state_get_type (void)
-{
-  static GType etype = 0;
-  if (etype == 0) {
-    static const GFlagsValue values[] = {
-      { GST_STATE_VOID_PENDING, "GST_STATE_VOID_PENDING", "void-pending" },
-      { GST_STATE_NULL, "GST_STATE_NULL", "null" },
-      { GST_STATE_READY, "GST_STATE_READY", "ready" },
-      { GST_STATE_PAUSED, "GST_STATE_PAUSED", "paused" },
-      { GST_STATE_PLAYING, "GST_STATE_PLAYING", "playing" },
-      { 0, NULL, NULL }
-    };
-    etype = g_flags_register_static ("GstElementState", values);
-  }
-  return etype;
-}
-
-GType
-gst_element_state_return_get_type (void)
-{
-  static GType etype = 0;
-  if (etype == 0) {
-    static const GEnumValue values[] = {
-      { GST_STATE_FAILURE, "GST_STATE_FAILURE", "failure" },
-      { GST_STATE_SUCCESS, "GST_STATE_SUCCESS", "success" },
-      { GST_STATE_ASYNC, "GST_STATE_ASYNC", "async" },
-      { 0, NULL, NULL }
-    };
-    etype = g_enum_register_static ("GstElementStateReturn", values);
-  }
-  return etype;
-}
-
-GType
-gst_result_get_type (void)
-{
-  static GType etype = 0;
-  if (etype == 0) {
-    static const GEnumValue values[] = {
-      { GST_RESULT_OK, "GST_RESULT_OK", "ok" },
-      { GST_RESULT_NOK, "GST_RESULT_NOK", "nok" },
-      { GST_RESULT_NOT_IMPL, "GST_RESULT_NOT_IMPL", "not-impl" },
-      { 0, NULL, NULL }
-    };
-    etype = g_enum_register_static ("GstResult", values);
-  }
-  return etype;
-}
-
-
 /* enumerations from "gstautoplug.h" */
 GType
 gst_autoplug_flags_get_type (void)
@@ -134,6 +82,23 @@ gst_buffer_flag_get_type (void)
 }
 
 
+/* enumerations from "gstcaps.h" */
+GType
+gst_caps_flags_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GFlagsValue values[] = {
+      { GST_CAPS_UNUSED, "GST_CAPS_UNUSED", "unused" },
+      { GST_CAPS_FLOATING, "GST_CAPS_FLOATING", "floating" },
+      { 0, NULL, NULL }
+    };
+    etype = g_flags_register_static ("GstCapsFlags", values);
+  }
+  return etype;
+}
+
+
 /* enumerations from "gstclock.h" */
 GType
 gst_clock_entry_status_get_type (void)
@@ -256,6 +221,8 @@ gst_element_flags_get_type (void)
       { GST_ELEMENT_USE_THREADSAFE_PROPERTIES, "GST_ELEMENT_USE_THREADSAFE_PROPERTIES", "use-threadsafe-properties" },
       { GST_ELEMENT_SCHEDULER_PRIVATE1, "GST_ELEMENT_SCHEDULER_PRIVATE1", "scheduler-private1" },
       { GST_ELEMENT_SCHEDULER_PRIVATE2, "GST_ELEMENT_SCHEDULER_PRIVATE2", "scheduler-private2" },
+      { GST_ELEMENT_LOCKED_STATE, "GST_ELEMENT_LOCKED_STATE", "locked-state" },
+      { GST_ELEMENT_ERROR, "GST_ELEMENT_ERROR", "error" },
       { GST_ELEMENT_FLAG_LAST, "GST_ELEMENT_FLAG_LAST", "flag-last" },
       { 0, NULL, NULL }
     };
@@ -277,7 +244,6 @@ gst_event_type_get_type (void)
       { GST_EVENT_FLUSH, "GST_EVENT_FLUSH", "flush" },
       { GST_EVENT_EMPTY, "GST_EVENT_EMPTY", "empty" },
       { GST_EVENT_DISCONTINUOUS, "GST_EVENT_DISCONTINUOUS", "discontinuous" },
-      { GST_EVENT_NEW_MEDIA, "GST_EVENT_NEW_MEDIA", "new-media" },
       { GST_EVENT_QOS, "GST_EVENT_QOS", "qos" },
       { GST_EVENT_SEEK, "GST_EVENT_SEEK", "seek" },
       { GST_EVENT_SEEK_SEGMENT, "GST_EVENT_SEEK_SEGMENT", "seek-segment" },
@@ -287,6 +253,7 @@ gst_event_type_get_type (void)
       { GST_EVENT_FILLER, "GST_EVENT_FILLER", "filler" },
       { GST_EVENT_TS_OFFSET, "GST_EVENT_TS_OFFSET", "ts-offset" },
       { GST_EVENT_INTERRUPT, "GST_EVENT_INTERRUPT", "interrupt" },
+      { GST_EVENT_NAVIGATION, "GST_EVENT_NAVIGATION", "navigation" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static ("GstEventType", values);
@@ -358,7 +325,6 @@ gst_format_get_type (void)
       { GST_FORMAT_TIME, "GST_FORMAT_TIME", "time" },
       { GST_FORMAT_BUFFERS, "GST_FORMAT_BUFFERS", "buffers" },
       { GST_FORMAT_PERCENT, "GST_FORMAT_PERCENT", "percent" },
-      { GST_FORMAT_UNITS, "GST_FORMAT_UNITS", "units" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static ("GstFormat", values);
@@ -466,6 +432,59 @@ gst_index_flags_get_type (void)
 }
 
 
+/* enumerations from "gstinfo.h" */
+GType
+gst_debug_level_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GST_LEVEL_NONE, "GST_LEVEL_NONE", "none" },
+      { GST_LEVEL_ERROR, "GST_LEVEL_ERROR", "error" },
+      { GST_LEVEL_WARNING, "GST_LEVEL_WARNING", "warning" },
+      { GST_LEVEL_INFO, "GST_LEVEL_INFO", "info" },
+      { GST_LEVEL_DEBUG, "GST_LEVEL_DEBUG", "debug" },
+      { GST_LEVEL_LOG, "GST_LEVEL_LOG", "log" },
+      { GST_LEVEL_COUNT, "GST_LEVEL_COUNT", "count" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("GstDebugLevel", values);
+  }
+  return etype;
+}
+
+GType
+gst_debug_color_flags_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GST_DEBUG_FG_BLACK, "GST_DEBUG_FG_BLACK", "fg-black" },
+      { GST_DEBUG_FG_RED, "GST_DEBUG_FG_RED", "fg-red" },
+      { GST_DEBUG_FG_GREEN, "GST_DEBUG_FG_GREEN", "fg-green" },
+      { GST_DEBUG_FG_YELLOW, "GST_DEBUG_FG_YELLOW", "fg-yellow" },
+      { GST_DEBUG_FG_BLUE, "GST_DEBUG_FG_BLUE", "fg-blue" },
+      { GST_DEBUG_FG_MAGENTA, "GST_DEBUG_FG_MAGENTA", "fg-magenta" },
+      { GST_DEBUG_FG_CYAN, "GST_DEBUG_FG_CYAN", "fg-cyan" },
+      { GST_DEBUG_FG_WHITE, "GST_DEBUG_FG_WHITE", "fg-white" },
+      { GST_DEBUG_BG_BLACK, "GST_DEBUG_BG_BLACK", "bg-black" },
+      { GST_DEBUG_BG_RED, "GST_DEBUG_BG_RED", "bg-red" },
+      { GST_DEBUG_BG_GREEN, "GST_DEBUG_BG_GREEN", "bg-green" },
+      { GST_DEBUG_BG_YELLOW, "GST_DEBUG_BG_YELLOW", "bg-yellow" },
+      { GST_DEBUG_BG_BLUE, "GST_DEBUG_BG_BLUE", "bg-blue" },
+      { GST_DEBUG_BG_MAGENTA, "GST_DEBUG_BG_MAGENTA", "bg-magenta" },
+      { GST_DEBUG_BG_CYAN, "GST_DEBUG_BG_CYAN", "bg-cyan" },
+      { GST_DEBUG_BG_WHITE, "GST_DEBUG_BG_WHITE", "bg-white" },
+      { GST_DEBUG_BOLD, "GST_DEBUG_BOLD", "bold" },
+      { GST_DEBUG_UNDERLINE, "GST_DEBUG_UNDERLINE", "underline" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("GstDebugColorFlags", values);
+  }
+  return etype;
+}
+
+
 /* enumerations from "gstpad.h" */
 GType
 gst_pad_link_return_get_type (void)
@@ -532,6 +551,21 @@ gst_pad_presence_get_type (void)
   return etype;
 }
 
+GType
+gst_pad_template_flags_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GST_PAD_TEMPLATE_FIXED, "GST_PAD_TEMPLATE_FIXED", "fixed" },
+      { GST_PAD_TEMPLATE_FLAG_LAST, "GST_PAD_TEMPLATE_FLAG_LAST", "flag-last" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("GstPadTemplateFlags", values);
+  }
+  return etype;
+}
+
 
 /* enumerations from "gstplugin.h" */
 GType
@@ -542,6 +576,7 @@ gst_plugin_error_get_type (void)
     static const GEnumValue values[] = {
       { GST_PLUGIN_ERROR_MODULE, "GST_PLUGIN_ERROR_MODULE", "module" },
       { GST_PLUGIN_ERROR_DEPENDENCIES, "GST_PLUGIN_ERROR_DEPENDENCIES", "dependencies" },
+      { GST_PLUGIN_ERROR_NAME_MISMATCH, "GST_PLUGIN_ERROR_NAME_MISMATCH", "name-mismatch" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static ("GstPluginError", values);
@@ -577,6 +612,21 @@ gst_props_type_get_type (void)
   return etype;
 }
 
+GType
+gst_props_flags_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GFlagsValue values[] = {
+      { GST_PROPS_FIXED, "GST_PROPS_FIXED", "fixed" },
+      { GST_PROPS_FLOATING, "GST_PROPS_FLOATING", "floating" },
+      { 0, NULL, NULL }
+    };
+    etype = g_flags_register_static ("GstPropsFlags", values);
+  }
+  return etype;
+}
+
 
 /* enumerations from "gstquery.h" */
 GType
@@ -642,9 +692,9 @@ gst_thread_state_get_type (void)
   static GType etype = 0;
   if (etype == 0) {
     static const GEnumValue values[] = {
-      { GST_THREAD_STATE_STARTED, "GST_THREAD_STATE_STARTED", "state-started" },
       { GST_THREAD_STATE_SPINNING, "GST_THREAD_STATE_SPINNING", "state-spinning" },
       { GST_THREAD_STATE_REAPING, "GST_THREAD_STATE_REAPING", "state-reaping" },
+      { GST_THREAD_MUTEX_LOCKED, "GST_THREAD_MUTEX_LOCKED", "mutex-locked" },
       { GST_THREAD_FLAG_LAST, "GST_THREAD_FLAG_LAST", "flag-last" },
       { 0, NULL, NULL }
     };
@@ -654,6 +704,95 @@ gst_thread_state_get_type (void)
 }
 
 
+/* enumerations from "gsttrace.h" */
+GType
+gst_alloc_trace_flags_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GFlagsValue values[] = {
+      { GST_ALLOC_TRACE_LIVE, "GST_ALLOC_TRACE_LIVE", "live" },
+      { GST_ALLOC_TRACE_MEM_LIVE, "GST_ALLOC_TRACE_MEM_LIVE", "mem-live" },
+      { 0, NULL, NULL }
+    };
+    etype = g_flags_register_static ("GstAllocTraceFlags", values);
+  }
+  return etype;
+}
+
+
+/* enumerations from "gsttypefind.h" */
+GType
+gst_type_find_probability_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GST_TYPE_FIND_MINIMUM, "GST_TYPE_FIND_MINIMUM", "minimum" },
+      { GST_TYPE_FIND_POSSIBLE, "GST_TYPE_FIND_POSSIBLE", "possible" },
+      { GST_TYPE_FIND_LIKELY, "GST_TYPE_FIND_LIKELY", "likely" },
+      { GST_TYPE_FIND_NEARLY_CERTAIN, "GST_TYPE_FIND_NEARLY_CERTAIN", "nearly-certain" },
+      { GST_TYPE_FIND_MAXIMUM, "GST_TYPE_FIND_MAXIMUM", "maximum" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("GstTypeFindProbability", values);
+  }
+  return etype;
+}
+
+
+/* enumerations from "gsttypes.h" */
+GType
+gst_element_state_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GFlagsValue values[] = {
+      { GST_STATE_VOID_PENDING, "GST_STATE_VOID_PENDING", "void-pending" },
+      { GST_STATE_NULL, "GST_STATE_NULL", "null" },
+      { GST_STATE_READY, "GST_STATE_READY", "ready" },
+      { GST_STATE_PAUSED, "GST_STATE_PAUSED", "paused" },
+      { GST_STATE_PLAYING, "GST_STATE_PLAYING", "playing" },
+      { 0, NULL, NULL }
+    };
+    etype = g_flags_register_static ("GstElementState", values);
+  }
+  return etype;
+}
+
+GType
+gst_element_state_return_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GST_STATE_FAILURE, "GST_STATE_FAILURE", "failure" },
+      { GST_STATE_SUCCESS, "GST_STATE_SUCCESS", "success" },
+      { GST_STATE_ASYNC, "GST_STATE_ASYNC", "async" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("GstElementStateReturn", values);
+  }
+  return etype;
+}
+
+GType
+gst_result_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GST_RESULT_OK, "GST_RESULT_OK", "ok" },
+      { GST_RESULT_NOK, "GST_RESULT_NOK", "nok" },
+      { GST_RESULT_NOT_IMPL, "GST_RESULT_NOT_IMPL", "not-impl" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("GstResult", values);
+  }
+  return etype;
+}
+
+
 /* enumerations from "gstregistry.h" */
 GType
 gst_registry_return_get_type (void)
@@ -703,6 +842,9 @@ gst_parse_error_get_type (void)
       { GST_PARSE_ERROR_NO_SUCH_ELEMENT, "GST_PARSE_ERROR_NO_SUCH_ELEMENT", "no-such-element" },
       { GST_PARSE_ERROR_NO_SUCH_PROPERTY, "GST_PARSE_ERROR_NO_SUCH_PROPERTY", "no-such-property" },
       { GST_PARSE_ERROR_LINK, "GST_PARSE_ERROR_LINK", "link" },
+      { GST_PARSE_ERROR_COULD_NOT_SET_PROPERTY, "GST_PARSE_ERROR_COULD_NOT_SET_PROPERTY", "could-not-set-property" },
+      { GST_PARSE_ERROR_EMPTY_BIN, "GST_PARSE_ERROR_EMPTY_BIN", "empty-bin" },
+      { GST_PARSE_ERROR_EMPTY, "GST_PARSE_ERROR_EMPTY", "empty" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static ("GstParseError", values);
