@@ -13,17 +13,6 @@ G_BEGIN_DECLS
 GType gst_object_flags_get_type (void);
 #define GST_TYPE_OBJECT_FLAGS (gst_object_flags_get_type())
 
-/* enumerations from "gsttypes.h" */
-
-GType gst_element_state_get_type (void);
-#define GST_TYPE_ELEMENT_STATE (gst_element_state_get_type())
-
-GType gst_element_state_return_get_type (void);
-#define GST_TYPE_ELEMENT_STATE_RETURN (gst_element_state_return_get_type())
-
-GType gst_result_get_type (void);
-#define GST_TYPE_RESULT (gst_result_get_type())
-
 /* enumerations from "gstautoplug.h" */
 
 GType gst_autoplug_flags_get_type (void);
@@ -39,19 +28,10 @@ GType gst_bin_flags_get_type (void);
 GType gst_buffer_flag_get_type (void);
 #define GST_TYPE_BUFFER_FLAG (gst_buffer_flag_get_type())
 
-/* enumerations from "gstcache.h" */
+/* enumerations from "gstcaps.h" */
 
-GType gst_cache_certainty_get_type (void);
-#define GST_TYPE_CACHE_CERTAINTY (gst_cache_certainty_get_type())
-
-GType gst_cache_entry_type_get_type (void);
-#define GST_TYPE_CACHE_ENTRY_TYPE (gst_cache_entry_type_get_type())
-
-GType gst_cache_lookup_method_get_type (void);
-#define GST_TYPE_CACHE_LOOKUP_METHOD (gst_cache_lookup_method_get_type())
-
-GType gst_assoc_flags_get_type (void);
-#define GST_TYPE_ASSOC_FLAGS (gst_assoc_flags_get_type())
+GType gst_caps_flags_get_type (void);
+#define GST_TYPE_CAPS_FLAGS (gst_caps_flags_get_type())
 
 /* enumerations from "gstclock.h" */
 
@@ -101,13 +81,38 @@ GType gst_seek_accuracy_get_type (void);
 GType gst_format_get_type (void);
 #define GST_TYPE_FORMAT (gst_format_get_type())
 
+/* enumerations from "gstindex.h" */
+
+GType gst_index_certainty_get_type (void);
+#define GST_TYPE_INDEX_CERTAINTY (gst_index_certainty_get_type())
+
+GType gst_index_entry_type_get_type (void);
+#define GST_TYPE_INDEX_ENTRY_TYPE (gst_index_entry_type_get_type())
+
+GType gst_index_lookup_method_get_type (void);
+#define GST_TYPE_INDEX_LOOKUP_METHOD (gst_index_lookup_method_get_type())
+
+GType gst_assoc_flags_get_type (void);
+#define GST_TYPE_ASSOC_FLAGS (gst_assoc_flags_get_type())
+
+GType gst_index_resolver_method_get_type (void);
+#define GST_TYPE_INDEX_RESOLVER_METHOD (gst_index_resolver_method_get_type())
+
+GType gst_index_flags_get_type (void);
+#define GST_TYPE_INDEX_FLAGS (gst_index_flags_get_type())
+
+/* enumerations from "gstinfo.h" */
+
+GType gst_debug_level_get_type (void);
+#define GST_TYPE_DEBUG_LEVEL (gst_debug_level_get_type())
+
+GType gst_debug_color_flags_get_type (void);
+#define GST_TYPE_DEBUG_COLOR_FLAGS (gst_debug_color_flags_get_type())
+
 /* enumerations from "gstpad.h" */
 
-GType gst_pad_connect_return_get_type (void);
-#define GST_TYPE_PAD_CONNECT_RETURN (gst_pad_connect_return_get_type())
-
-GType gst_pad_query_type_get_type (void);
-#define GST_TYPE_PAD_QUERY_TYPE (gst_pad_query_type_get_type())
+GType gst_pad_link_return_get_type (void);
+#define GST_TYPE_PAD_LINK_RETURN (gst_pad_link_return_get_type())
 
 GType gst_pad_direction_get_type (void);
 #define GST_TYPE_PAD_DIRECTION (gst_pad_direction_get_type())
@@ -118,6 +123,9 @@ GType gst_pad_flags_get_type (void);
 GType gst_pad_presence_get_type (void);
 #define GST_TYPE_PAD_PRESENCE (gst_pad_presence_get_type())
 
+GType gst_pad_template_flags_get_type (void);
+#define GST_TYPE_PAD_TEMPLATE_FLAGS (gst_pad_template_flags_get_type())
+
 /* enumerations from "gstplugin.h" */
 
 GType gst_plugin_error_get_type (void);
@@ -128,6 +136,14 @@ GType gst_plugin_error_get_type (void);
 GType gst_props_type_get_type (void);
 #define GST_TYPE_PROPS_TYPE (gst_props_type_get_type())
 
+GType gst_props_flags_get_type (void);
+#define GST_TYPE_PROPS_FLAGS (gst_props_flags_get_type())
+
+/* enumerations from "gstquery.h" */
+
+GType gst_query_type_get_type (void);
+#define GST_TYPE_QUERY_TYPE (gst_query_type_get_type())
+
 /* enumerations from "gstscheduler.h" */
 
 GType gst_scheduler_flags_get_type (void);
@@ -141,6 +157,27 @@ GType gst_scheduler_state_get_type (void);
 GType gst_thread_state_get_type (void);
 #define GST_TYPE_THREAD_STATE (gst_thread_state_get_type())
 
+/* enumerations from "gsttrace.h" */
+
+GType gst_alloc_trace_flags_get_type (void);
+#define GST_TYPE_ALLOC_TRACE_FLAGS (gst_alloc_trace_flags_get_type())
+
+/* enumerations from "gsttypefind.h" */
+
+GType gst_type_find_probability_get_type (void);
+#define GST_TYPE_TYPE_FIND_PROBABILITY (gst_type_find_probability_get_type())
+
+/* enumerations from "gsttypes.h" */
+
+GType gst_element_state_get_type (void);
+#define GST_TYPE_ELEMENT_STATE (gst_element_state_get_type())
+
+GType gst_element_state_return_get_type (void);
+#define GST_TYPE_ELEMENT_STATE_RETURN (gst_element_state_return_get_type())
+
+GType gst_result_get_type (void);
+#define GST_TYPE_RESULT (gst_result_get_type())
+
 /* enumerations from "gstregistry.h" */
 
 GType gst_registry_return_get_type (void);
