@@ -20,13 +20,19 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* #define GST_DEBUG_ENABLED */
 #include "gst_private.h"
 
-#include "gstlog.h"
+#include "gstinfo.h"
 #include "gstprops.h"
 #include "gstmemchunk.h"
 
+#ifndef GST_DISABLE_TRACE
+/* #define GST_WITH_ALLOC_TRACE */
+#include "gsttrace.h"
+static GstAllocTrace *_props_trace;
+static GstAllocTrace *_entries_trace;
+#endif
+
 GType _gst_props_type;
 GType _gst_props_entry_type;
 
@@ -64,52 +70,494 @@ struct _GstPropsEntry {
 static GstMemChunk *_gst_props_entries_chunk;
 static GstMemChunk *_gst_props_chunk;
 
+/* transform functions */
+static void		gst_props_transform_to_string		(const GValue *src_value, GValue *dest_value);
+static gchar *		gst_props_entry_to_string		(GstPropsEntry *entry);
+gboolean		__gst_props_parse_string		(gchar *r, gchar **end, gchar **next);
+
 static gboolean 	gst_props_entry_check_compatibility 	(GstPropsEntry *entry1, GstPropsEntry *entry2);
 static GList* 		gst_props_list_copy 			(GList *propslist);
 
-static void
-transform_func (const GValue *src_value,
-		GValue *dest_value)
+static GstPropsEntry*	gst_props_alloc_entry			(void);
+static inline void	gst_props_entry_free			(GstPropsEntry *entry);
+
+static void		gst_props_destroy			(GstProps *props);
+
+static gchar *
+gst_props_entry_to_string (GstPropsEntry *entry)
 {
-  GstProps *props = g_value_peek_pointer (src_value);
-  GString *result = g_string_new ("");
-
-  if (props) {
-    GList *propslist = props->properties;
-
-    while (propslist) {
-      GstPropsEntry *entry = (GstPropsEntry *)propslist->data;
-      const gchar *name = g_quark_to_string (entry->propid);
-
-      switch (entry->propstype) {
-        case GST_PROPS_INT_TYPE:
-  	  g_string_append_printf (result, "%s=(int) %d", name, entry->data.int_data);
-	  break;
-        case GST_PROPS_FLOAT_TYPE:
-  	  g_string_append_printf (result, "%s=(float) %f", name, entry->data.float_data);
-	  break;
-        case GST_PROPS_FOURCC_TYPE:
-  	  g_string_append_printf (result, "%s=(fourcc) '%4.4s'", name, (gchar *)&entry->data.fourcc_data);
-	  break;
-        case GST_PROPS_BOOLEAN_TYPE:
-  	  g_string_append_printf (result, "%s=(boolean) %s", name, 
-			  	  (entry->data.bool_data ? "TRUE" : "FALSE"));
-	  break;
-        case GST_PROPS_STRING_TYPE:
-	  g_string_append_printf (result, "%s=(string) '%s'", name, entry->data.string_data.string);
-	  break;
-        default:
-	  break;
-      }
-
-      propslist = g_list_next (propslist);
-      if (propslist) {
-        g_string_append (result, "; ");
+  switch (entry->propstype) {
+    case GST_PROPS_INT_TYPE:
+      return g_strdup_printf ("int = %d", entry->data.int_data);
+    case GST_PROPS_FLOAT_TYPE:
+      return g_strdup_printf ("float = %g", entry->data.float_data);
+      break;
+    case GST_PROPS_FOURCC_TYPE: {
+      gchar fourcc[5] = { GST_FOURCC_ARGS (entry->data.fourcc_data), '\0' }; /* Do all compilers understand that? */
+      if (g_ascii_isalnum(fourcc[1]) && g_ascii_isalnum(fourcc[2]) &&
+	  g_ascii_isalnum(fourcc[3]) && g_ascii_isalnum(fourcc[4])) {
+        return g_strdup_printf ("fourcc = %s", fourcc);
+      } else {
+        return g_strdup_printf ("fourcc = %d", entry->data.fourcc_data);
       }
     }
+    case GST_PROPS_BOOLEAN_TYPE:
+      return g_strdup_printf ("bool = %s", (entry->data.bool_data ? "TRUE" : "FALSE"));
+    case GST_PROPS_STRING_TYPE:
+      /* FIXME: Need to escape stuff here */
+      return g_strdup_printf ("string = '%s'", entry->data.string_data.string);
+    case GST_PROPS_INT_RANGE_TYPE:
+      return g_strdup_printf ("int = [%d, %d]", entry->data.int_range_data.min, entry->data.int_range_data.max);
+    case GST_PROPS_FLOAT_RANGE_TYPE:
+      return g_strdup_printf ("float = [%g, %g]", entry->data.float_range_data.min, entry->data.float_range_data.max);
+    case GST_PROPS_LIST_TYPE: {
+      GList *walk;
+      GString *s;
+      gchar *temp;
+      
+      s = g_string_new ("list = (");
+      walk = entry->data.list_data.entries;
+      while (walk) {
+	temp = gst_props_entry_to_string ((GstPropsEntry *) walk->data);
+	g_string_append (s, temp);
+	walk = walk->next;
+	if (walk) {
+	  g_string_append (s, ", ");
+	} else {
+	  g_string_append (s, ")");
+	}
+	g_free (temp);
+      }
+      temp = s->str;
+      g_string_free (s, FALSE);
+      return temp;
+    }
+    default:
+      /* transforms always succeed */
+      g_warning("corrupt property list, type==%d\n",entry->propstype);
+      g_assert_not_reached();
+      return NULL;
   }
-  dest_value->data[0].v_pointer = result->str;
-  g_string_free (result, FALSE);
+}
+/**
+ * gst_props_to_string:
+ * props: the props to convert to a string
+ *
+ * Converts a #GstProps into a readable format. This is mainly intended for
+ * debugging purposes. You have to free the string using g_free.
+ * A string converted with #gst_props_to_string can always be converted back to
+ * its props representation using #gst_props_from_string.
+ *
+ * Returns: A newly allocated string
+ */
+gchar *
+gst_props_to_string (GstProps *props)
+{
+  GString *s;
+  gchar *temp;
+  GList *propslist; 
+
+  s = g_string_new ("");
+  propslist = props->properties;
+  while (propslist) {
+    GstPropsEntry *entry = (GstPropsEntry *)propslist->data;
+    const gchar *name = g_quark_to_string (entry->propid);    
+  
+    temp = gst_props_entry_to_string (entry);
+    propslist = g_list_next (propslist);
+    if (temp) {
+      if (propslist) {
+        g_string_append_printf (s, "%s:%s, ", name, temp);
+      } else {
+        g_string_append_printf (s, "%s:%s", name, temp);
+      }
+      g_free (temp);
+    }
+  }
+  temp = s->str;
+  g_string_free (s, FALSE);
+
+  return temp;
+}
+static void
+gst_props_transform_to_string (const GValue *src_value, GValue *dest_value)
+{
+  GstProps *props = g_value_peek_pointer (src_value);
+
+  if (props)
+    dest_value->data[0].v_pointer = gst_props_to_string (props);
+}
+/*
+ * r will still point to the string. if end == next, the string will not be 
+ * null-terminated. In all other cases it will be.
+ * end = pointer to char behind end of string, next = pointer to start of
+ * unread data.
+ * THIS FUNCTION MODIFIES THE STRING AND DETECTS INSIDE A NONTERMINATED STRING 
+ */
+gboolean
+__gst_props_parse_string (gchar *r, gchar **end, gchar **next)
+{
+  gchar *w;
+  gchar c = '\0';
+
+  w = r;
+  if (*r == '\'' || *r == '\"') {
+    c = *r;
+    r++;
+  }
+  
+  for (;;r++) {
+    if (*r == '\0') {
+      if (c) {
+        goto error;
+      } else {
+        goto found;
+      }
+    }
+    
+    if (*r == '\\') {
+      r++;
+      if (*r == '\0')
+	goto error;
+      *w++ = *r;
+      continue;
+    }
+    
+    if (*r == c) {
+      r++;
+      if (*r == '\0')
+	goto found;
+      break;
+    }
+
+    if (!c) {
+      if (g_ascii_isspace (*r))
+	break;
+      /* this needs to be escaped */
+      if (*r == ',' || *r == ')' || *r == ']' || *r == ':' ||
+	  *r == ';' || *r == '(' || *r == '[')
+        break;
+    }
+    *w++ = *r;
+  }
+
+found:
+  while (g_ascii_isspace (*r)) r++;
+  if (w != r)
+    *w++ = '\0';
+  *end = w;
+  *next = r;
+  return TRUE;
+
+error:
+  return FALSE;
+}
+static GstPropsEntry *
+gst_props_entry_from_string_no_name (gchar *s, gchar **after, gboolean has_type)
+{
+  GstPropsEntry *entry;
+  gchar org = '\0';
+  gchar *end, *next, *check = s;
+  GstPropsType type = GST_PROPS_INVALID_TYPE;
+  /* [TYPE=]VALUE */
+  /*
+   * valid type identifiers case insensitive:
+   * INT: "i", "int"
+   * FLOAT: "f", "float"
+   * FOURCC: "4", "fourcc"
+   * BOOLEAN: "b", "bool", "boolean"
+   * STRING: "s", "str", "string"
+   * lists/ranges are identified by the value
+   */
+  if (g_ascii_tolower (s[0]) == 'i') {
+    type = GST_PROPS_INT_TYPE;
+    if (g_ascii_tolower (s[1]) == 'n' && g_ascii_tolower (s[2]) == 't') {
+      check = s + 3;
+    } else {
+      check = s + 1;
+    }
+  } else if (g_ascii_tolower (s[0]) == 'f') {
+    type = GST_PROPS_FLOAT_TYPE;
+    if (g_ascii_tolower (s[1]) == 'l' && g_ascii_tolower (s[2]) == 'o' &&
+        g_ascii_tolower (s[3]) == 'a' && g_ascii_tolower (s[4]) == 't') {
+      check = s + 5;
+    } else if (g_ascii_tolower (s[1]) == 'o' && g_ascii_tolower (s[2]) == 'u' &&
+               g_ascii_tolower (s[3]) == 'r' && g_ascii_tolower (s[4]) == 'c' && 
+               g_ascii_tolower (s[5]) == 'c') {
+      check = s + 6;
+      type = GST_PROPS_FOURCC_TYPE;
+    } else {
+      check = s + 1;
+    }
+  } else if (g_ascii_tolower (s[0]) == '4') {
+    type = GST_PROPS_FOURCC_TYPE;
+    check = s + 1;
+  } else if (g_ascii_tolower (s[0]) == 'b') {
+    type = GST_PROPS_BOOLEAN_TYPE;
+    if (g_ascii_tolower (s[1]) == 'o' && g_ascii_tolower (s[2]) == 'o' &&
+        g_ascii_tolower (s[3]) == 'l') {
+      if (g_ascii_tolower (s[4]) == 'e' && g_ascii_tolower (s[5]) == 'a' &&
+          g_ascii_tolower (s[6]) == 'n') {
+	check = s + 7;
+      } else {
+	check = s + 4;
+      }
+    } else {
+      check = s + 1;
+    }
+  } else if (g_ascii_tolower (s[0]) == 's') {
+    type = GST_PROPS_STRING_TYPE;
+    if (g_ascii_tolower (s[1]) == 't' && g_ascii_tolower (s[2]) == 'r') {
+      if (g_ascii_tolower (s[3]) == 'i' && g_ascii_tolower (s[4]) == 'n' &&
+          g_ascii_tolower (s[5]) == 'g') {
+	check = s + 6;
+      } else {
+	check = s + 3;
+      }
+    } else {
+      check = s + 1;
+    }
+  } else if (g_ascii_tolower (s[0]) == 'l') {
+    type = GST_PROPS_LIST_TYPE;
+    if (g_ascii_tolower (s[1]) == 'i' && g_ascii_tolower (s[2]) == 's' &&
+        g_ascii_tolower (s[3]) == 't') {
+      check = s + 4;
+    } else {
+      check = s + 1;
+    }
+  }
+  while (g_ascii_isspace(*check)) check++;
+  if (*check != '=') {
+    if (has_type) goto error;
+    type = GST_PROPS_INVALID_TYPE;
+    check = s;
+  } else {
+    check++;
+    while (g_ascii_isspace(*check)) check++;
+  }
+  /* ok. Now type is GST_PROPS_INVALID_TYPE for guessing or the selected type.
+     check points to the string containing the contents. s still is the beginning
+     of the string */
+  if (type == GST_PROPS_INVALID_TYPE || type == GST_PROPS_INT_TYPE || type == GST_PROPS_FOURCC_TYPE) {
+    glong l;
+    l = strtol (check, &end, 0);
+    while (g_ascii_isspace (*end)) end++;
+    if (*end == '\0' || *end == ',' || *end == ';' || *end == ')' || *end == ']') {
+      *after = end;
+      entry = gst_props_alloc_entry ();
+      if (type == GST_PROPS_FOURCC_TYPE) {
+        entry->propstype = GST_PROPS_FOURCC_TYPE;
+        entry->data.fourcc_data = l;
+      } else {
+        entry->propstype = GST_PROPS_INT_TYPE;
+        entry->data.int_data = l;
+      }
+      return entry;
+    }
+  }
+  if (type == GST_PROPS_INVALID_TYPE || type == GST_PROPS_FLOAT_TYPE) {
+    gdouble d;
+    d = g_ascii_strtod (check, &end);
+    while (g_ascii_isspace (*end)) end++;
+    if (*end == '\0' || *end == ',' || *end == ';' || *end == ')' || *end == ']') {
+      *after = end;
+      entry = gst_props_alloc_entry ();
+      entry->propstype = GST_PROPS_FLOAT_TYPE;
+      entry->data.float_data = d;
+      return entry;
+    }
+  }
+  if ((type == GST_PROPS_INVALID_TYPE || type == GST_PROPS_FLOAT_TYPE ||
+       type == GST_PROPS_INT_TYPE) && *check == '[') {
+    GstPropsEntry *min, *max;
+    check++;
+    if (g_ascii_isspace (*check)) check++;
+    min = gst_props_entry_from_string_no_name (check, &check, FALSE);
+    if (!min) goto error;
+    if (*check++ != ',') goto error;
+    if (g_ascii_isspace (*check)) check++;
+    max = gst_props_entry_from_string_no_name (check, &check, FALSE);
+    if (!max || *check++ != ']') {
+      gst_props_entry_destroy (min);
+      goto error;
+    }
+    if (g_ascii_isspace (*check)) check++;
+    entry = gst_props_alloc_entry ();
+    entry->propstype = GST_PROPS_FLOAT_RANGE_TYPE;
+    if (min->propstype == GST_PROPS_INT_TYPE && max->propstype == GST_PROPS_INT_TYPE && type != GST_PROPS_FLOAT_TYPE) {
+      entry->propstype = GST_PROPS_INT_RANGE_TYPE;
+      entry->data.int_range_data.min = min->data.int_data;
+      entry->data.int_range_data.max = max->data.int_data;
+    } else if (min->propstype == GST_PROPS_INT_TYPE && max->propstype == GST_PROPS_INT_TYPE && type == GST_PROPS_FLOAT_TYPE) {
+      entry->data.float_range_data.min = min->data.int_data;
+      entry->data.float_range_data.max = max->data.int_data;
+    } else if (min->propstype == GST_PROPS_INT_TYPE && max->propstype == GST_PROPS_FLOAT_TYPE && type != GST_PROPS_INT_TYPE) {
+      entry->data.float_range_data.min = min->data.int_data;
+      entry->data.float_range_data.max = max->data.float_data;
+    } else if (min->propstype == GST_PROPS_FLOAT_TYPE && max->propstype == GST_PROPS_INT_TYPE && type != GST_PROPS_INT_TYPE) {
+      entry->data.float_range_data.min = min->data.float_data;
+      entry->data.float_range_data.max = max->data.int_data;
+    } else if (min->propstype == GST_PROPS_FLOAT_TYPE && max->propstype == GST_PROPS_FLOAT_TYPE && type != GST_PROPS_INT_TYPE) {
+      entry->data.float_range_data.min = min->data.float_data;
+      entry->data.float_range_data.max = max->data.float_data;
+    } else {
+      gst_props_entry_destroy (min);
+      gst_props_entry_destroy (max);
+      gst_props_entry_destroy (entry);
+      goto error;
+    }
+    gst_props_entry_destroy (min);
+    gst_props_entry_destroy (max);
+    *after = check;
+    return entry;    
+  }
+  if ((type == GST_PROPS_INVALID_TYPE || type == GST_PROPS_LIST_TYPE) && *check == '(') {
+    GstPropsEntry *next;
+    check++;
+    entry = gst_props_alloc_entry ();
+    entry->propstype = GST_PROPS_LIST_TYPE;
+    entry->data.list_data.entries = NULL;
+    do {
+      while (g_ascii_isspace (*check)) check++;
+      next = gst_props_entry_from_string_no_name (check, &check, FALSE);
+      /* use g_list_append to keep original order */
+      entry->data.list_data.entries = g_list_append (entry->data.list_data.entries, next);
+      if (*check == ')') break;
+      if (*check++ != ',') goto error;
+    } while (TRUE);
+    check++;
+    while (g_ascii_isspace (*check)) check++;    
+    *after = check;
+    return entry;    
+  }
+  if (!__gst_props_parse_string (check, &next, &end))
+    goto error;
+  if (next == end) {
+    org = *end;
+    *end = '\0';
+  }
+  if (type == GST_PROPS_INVALID_TYPE || type == GST_PROPS_BOOLEAN_TYPE) {
+    if (!(g_ascii_strcasecmp (check, "true") &&
+          g_ascii_strcasecmp (check, "yes"))) {
+      entry = gst_props_alloc_entry ();
+      entry->propstype = GST_PROPS_BOOLEAN_TYPE;
+      entry->data.bool_data = TRUE;
+      goto string_out;
+    }
+    if (!(g_ascii_strcasecmp (check, "false") &&
+          g_ascii_strcasecmp (check, "no"))) {
+      entry = gst_props_alloc_entry ();
+      entry->propstype = GST_PROPS_BOOLEAN_TYPE;
+      entry->data.bool_data = FALSE;
+      goto string_out;
+    }
+  }
+  if (type == GST_PROPS_FOURCC_TYPE) {
+    gint l = strlen (check);
+    entry = gst_props_alloc_entry ();
+    entry->propstype = GST_PROPS_FOURCC_TYPE;
+    entry->data.fourcc_data = GST_MAKE_FOURCC(l > 0 ? check[0] : ' ',
+					      l > 1 ? check[1] : ' ',
+					      l > 2 ? check[2] : ' ',
+					      l > 3 ? check[3] : ' ');
+    goto string_out;
+  }
+  if (type == GST_PROPS_INVALID_TYPE || type == GST_PROPS_STRING_TYPE) {
+    entry = gst_props_alloc_entry ();
+    entry->propstype = GST_PROPS_STRING_TYPE;
+    entry->data.string_data.string = g_strdup (check);
+    goto string_out;
+  }
+error:
+  return NULL;
+
+string_out:
+  *next = org;
+  *after = end;
+  return entry;
+}
+static GstPropsEntry *
+gst_props_entry_from_string (gchar *str, gchar **after)
+{
+  /* NAME[:TYPE]=VALUE */
+  gchar *name;
+  gchar *s, *del;
+  gboolean has_type = FALSE;
+  GstPropsEntry *entry;
+
+  name = s = str;
+  while (g_ascii_isalnum (*s) || *s == '_' || *s == '-') s++;
+  del = s;
+  while (g_ascii_isspace (*s)) s++;
+  if (!(*s == '=' || *s == ':')) return NULL;  
+  if (*s == ':') has_type = TRUE;
+  s++;
+  while (g_ascii_isspace (*s)) s++;
+  *del = '\0';
+  
+  entry = gst_props_entry_from_string_no_name (s, &s, has_type);
+  if (entry) {
+    entry->propid = g_quark_from_string (name);
+    *after = s;
+  }
+  
+  return entry;
+}
+GstProps *
+__gst_props_from_string_func (gchar *s, gchar **end, gboolean caps)
+{
+  GstProps *props;
+  GstPropsEntry *entry;
+
+  props = gst_props_empty_new ();
+  for (;;) {
+    entry = gst_props_entry_from_string (s, &s);
+    if (!entry) goto error;
+    gst_props_add_entry (props, entry);
+    while (g_ascii_isspace (*s)) s++;
+    if ((*s == '\0') || /* end */
+        (caps && (*s == ';'))) /* another caps */
+      break;
+    if (!(*s == ',')) goto error;
+    s++;
+    while (g_ascii_isspace (*s)) s++;    
+  }
+
+  *end = s;
+  return props;  
+error:
+  gst_props_unref (props);
+  return NULL;
+}
+/**
+ * gst_props_from_string:
+ * str: the str to convert into props
+ *
+ * Tries to convert a string into a #GstProps. This is mainly intended for
+ * debugging purposes. The returned props are floating.
+ *
+ * Returns: A floating props or NULL if the string couldn't be converted
+ */
+GstProps *
+gst_props_from_string (gchar *str)
+{
+  /*
+   * format to parse is ENTRY[,ENTRY ...]
+   * ENTRY is NAME[:TYPE]=VALUE
+   * NAME is alphanumeric
+   * TYPE is a list of values
+   * VALUE is evil, see gst_props_entry_to_string
+   */
+  GstProps *props;
+  gchar *temp, *org;
+
+  g_return_val_if_fail (str != NULL, NULL);
+
+  org = g_strdup (str);
+  props = __gst_props_from_string_func (org, &temp, FALSE);
+  g_free (org);
+
+  return props;
 }
 
 void
@@ -127,50 +575,55 @@ _gst_props_initialize (void)
 		                       (GBoxedCopyFunc) gst_props_ref,
 		                       (GBoxedFreeFunc) gst_props_unref);
 
-  g_value_register_transform_func (_gst_props_type,
-                                   G_TYPE_STRING,
-                                   transform_func);
+  g_value_register_transform_func (_gst_props_type, G_TYPE_STRING,
+                                   gst_props_transform_to_string);
 
   _gst_props_entry_type = g_boxed_type_register_static ("GstPropsEntry",
-	                    (GBoxedCopyFunc) gst_props_entry_copy,
-	                    (GBoxedFreeFunc) gst_props_entry_destroy);
+		  (GBoxedCopyFunc) gst_props_entry_copy,
+		  (GBoxedFreeFunc) gst_props_entry_destroy);
+
+#ifndef GST_DISABLE_TRACE
+  _props_trace = gst_alloc_trace_register (GST_PROPS_TRACE_NAME);
+  _entries_trace = gst_alloc_trace_register (GST_PROPS_ENTRY_TRACE_NAME);
+#endif
 }
 
 static void
 gst_props_debug_entry (GstPropsEntry *entry)
 {
-  const gchar *name = g_quark_to_string (entry->propid);
+  /* unused when debugging is disabled */
+  G_GNUC_UNUSED const gchar *name = g_quark_to_string (entry->propid);
 
   switch (entry->propstype) {
     case GST_PROPS_INT_TYPE:
-      GST_DEBUG (GST_CAT_PROPERTIES, "%p: %s: int %d", entry, name, entry->data.int_data);
+      GST_CAT_DEBUG (GST_CAT_PROPERTIES, "%p: %s: int %d", entry, name, entry->data.int_data);
       break;
     case GST_PROPS_FLOAT_TYPE:
-      GST_DEBUG (GST_CAT_PROPERTIES, "%p: %s: float %f", entry, name, entry->data.float_data);
+      GST_CAT_DEBUG (GST_CAT_PROPERTIES, "%p: %s: float %g", entry, name, entry->data.float_data);
       break;
     case GST_PROPS_FOURCC_TYPE:
-      GST_DEBUG (GST_CAT_PROPERTIES, "%p: %s: fourcc %c%c%c%c", entry, name,
+      GST_CAT_DEBUG (GST_CAT_PROPERTIES, "%p: %s: fourcc %c%c%c%c", entry, name,
 	(entry->data.fourcc_data>>0)&0xff,
 	(entry->data.fourcc_data>>8)&0xff,
 	(entry->data.fourcc_data>>16)&0xff,
 	(entry->data.fourcc_data>>24)&0xff);
       break;
     case GST_PROPS_BOOLEAN_TYPE:
-      GST_DEBUG (GST_CAT_PROPERTIES, "%p: %s: bool %d", entry, name, entry->data.bool_data);
+      GST_CAT_DEBUG (GST_CAT_PROPERTIES, "%p: %s: bool %d", entry, name, entry->data.bool_data);
       break;
     case GST_PROPS_STRING_TYPE:
-      GST_DEBUG (GST_CAT_PROPERTIES, "%p: %s: string \"%s\"", entry, name, entry->data.string_data.string);
+      GST_CAT_DEBUG (GST_CAT_PROPERTIES, "%p: %s: string \"%s\"", entry, name, entry->data.string_data.string);
       break;
     case GST_PROPS_INT_RANGE_TYPE:
-      GST_DEBUG (GST_CAT_PROPERTIES, "%p: %s: int range %d-%d", entry, name, entry->data.int_range_data.min,
+      GST_CAT_DEBUG (GST_CAT_PROPERTIES, "%p: %s: int range %d-%d", entry, name, entry->data.int_range_data.min,
 		      entry->data.int_range_data.max);
       break;
     case GST_PROPS_FLOAT_RANGE_TYPE:
-      GST_DEBUG (GST_CAT_PROPERTIES, "%p: %s: float range %f-%f", entry, name, entry->data.float_range_data.min,
+      GST_CAT_DEBUG (GST_CAT_PROPERTIES, "%p: %s: float range %g-%g", entry, name, entry->data.float_range_data.min,
 		      entry->data.float_range_data.max);
       break;
     case GST_PROPS_LIST_TYPE:
-      GST_DEBUG (GST_CAT_PROPERTIES, "%p: [list]", entry);
+      GST_CAT_DEBUG (GST_CAT_PROPERTIES, "%p: [list]", entry);
       g_list_foreach (entry->data.list_data.entries, (GFunc) gst_props_debug_entry, NULL);
       break;
     default:
@@ -202,10 +655,70 @@ props_find_func (gconstpointer a,
 /* This is implemented as a huge macro because we cannot pass
  * va_list variables by reference on some architectures.
  */
+static inline GstPropsType
+gst_props_type_sanitize (GstPropsType type)
+{
+  switch (type) {
+    case GST_PROPS_INT_TYPE:
+    case GST_PROPS_INT_RANGE_TYPE:
+      return GST_PROPS_INT_TYPE;
+    case GST_PROPS_FLOAT_TYPE:
+    case GST_PROPS_FLOAT_RANGE_TYPE:
+      return GST_PROPS_FLOAT_TYPE;
+    case GST_PROPS_FOURCC_TYPE:
+    case GST_PROPS_BOOLEAN_TYPE:
+    case GST_PROPS_STRING_TYPE:
+      return type;
+    case GST_PROPS_LIST_TYPE:
+    case GST_PROPS_GLIST_TYPE:
+      return GST_PROPS_LIST_TYPE;
+    case GST_PROPS_END_TYPE:
+    case GST_PROPS_INVALID_TYPE:
+    case GST_PROPS_VAR_TYPE:
+    case GST_PROPS_LAST_TYPE:
+      break;
+  }
+  g_assert_not_reached ();
+  return GST_PROPS_END_TYPE;
+}
 #define GST_PROPS_ENTRY_FILL(entry, var_args)	 				\
 G_STMT_START { 									\
   entry->propstype = va_arg (var_args, GstPropsType); 				\
-										\
+  if (entry->propstype == GST_PROPS_LIST_TYPE) {				\
+    GList *_list = NULL;							\
+    GstPropsEntry *_cur = NULL; /* initialize so gcc doesn't complain */	\
+    GstPropsType _cur_type;							\
+    GstPropsType _type = va_arg (var_args, GstPropsType);			\
+    _cur_type = _type;				  				\
+    _type = gst_props_type_sanitize (_type);		      			\
+    while (_cur_type != GST_PROPS_END_TYPE) {				      	\
+      _cur = gst_props_alloc_entry ();						\
+      _cur->propid = entry->propid;						\
+      _cur->propstype = _cur_type;				      		\
+      _cur_type = gst_props_type_sanitize (_cur_type);			    	\
+      g_assert (_cur_type == _type);						\
+      GST_PROPS_ENTRY_FILL_DATA(_cur, var_args);				\
+      if (_cur_type == GST_PROPS_INT_TYPE) {					\
+	_list = gst_props_add_to_int_list (_list, _cur);      			\
+      } else {									\
+	_list = g_list_prepend (_list, _cur);					\
+      }										\
+      _cur_type = va_arg (var_args, GstPropsType);				\
+    }										\
+    if (_list && g_list_next (_list)) {							\
+      entry->data.list_data.entries = _list;				  	\
+    } else {									\
+      entry->propstype = _cur->propstype;					\
+      entry->data = _cur->data;							\
+      gst_props_entry_free (_cur);						\
+    }										\
+  } else {									\
+    GST_PROPS_ENTRY_FILL_DATA(entry, var_args);					\
+  }										\
+} G_STMT_END
+
+#define GST_PROPS_ENTRY_FILL_DATA(entry, var_args)	 			\
+G_STMT_START { 									\
   switch (entry->propstype) {							\
     case GST_PROPS_INT_TYPE:							\
       entry->data.int_data = va_arg (var_args, gint);				\
@@ -234,7 +747,10 @@ G_STMT_START { 									\
       entry->propstype = GST_PROPS_LIST_TYPE;					\
       entry->data.list_data.entries = g_list_copy (va_arg (var_args, GList*));	\
       break;									\
+    case GST_PROPS_END_TYPE:							\
+      break;									\
     default:									\
+      g_warning("attempt to set invalid props type\n");				\
       break;									\
   }										\
 } G_STMT_END
@@ -279,6 +795,8 @@ G_STMT_START { 									\
       case GST_PROPS_LIST_TYPE:							\
         *(va_arg (var_args, GList**)) = entry->data.list_data.entries;		\
         break;									\
+      case GST_PROPS_END_TYPE:							\
+        break;									\
       default:									\
         *result = FALSE;							\
         break;									\
@@ -292,8 +810,11 @@ gst_props_alloc_entry (void)
   GstPropsEntry *entry;
 
   entry = gst_mem_chunk_alloc (_gst_props_entries_chunk);
+#ifndef GST_DISABLE_TRACE
+  gst_alloc_trace_new (_entries_trace, entry);
+#endif
 
-  GST_DEBUG (GST_CAT_PROPERTIES, "new entry %p", entry);
+  GST_CAT_LOG (GST_CAT_PROPERTIES, "new entry %p", entry);
 
   return entry;
 }
@@ -313,7 +834,17 @@ gst_props_entry_clean (GstPropsEntry *entry)
       break;
   }
 }
-
+static inline void
+gst_props_entry_free (GstPropsEntry *entry)
+{
+#ifdef USE_POISONING
+  memset (entry, 0xff, sizeof(*entry));
+#endif
+  gst_mem_chunk_free (_gst_props_entries_chunk, entry);
+#ifndef GST_DISABLE_TRACE
+  gst_alloc_trace_free (_entries_trace, entry);
+#endif
+}
 /**
  * gst_props_entry_destroy:
  * @entry: the entry to destroy
@@ -323,11 +854,13 @@ gst_props_entry_clean (GstPropsEntry *entry)
 void
 gst_props_entry_destroy (GstPropsEntry *entry)
 {
-  GST_DEBUG (GST_CAT_PROPERTIES, "destroy entry %p", entry);
+  if (!entry) return;
+  
+  GST_CAT_LOG (GST_CAT_PROPERTIES, "destroy entry %p", entry);
 
   gst_props_entry_clean (entry);
 
-  gst_mem_chunk_free (_gst_props_entries_chunk, entry);
+  gst_props_entry_free (entry);
 }
 
 GType
@@ -349,8 +882,11 @@ gst_props_empty_new (void)
   GstProps *props;
 
   props = gst_mem_chunk_alloc (_gst_props_chunk);
+#ifndef GST_DISABLE_TRACE
+  gst_alloc_trace_new (_props_trace, props);
+#endif
 
-  GST_DEBUG (GST_CAT_PROPERTIES, "new %p", props);
+  GST_CAT_LOG (GST_CAT_PROPERTIES, "new %p", props);
 
   props->properties = NULL;
   props->refcount = 1;
@@ -433,7 +969,8 @@ gst_props_remove_entry_by_id (GstProps *props, GQuark propid)
 
     if (lentry->propid == propid) {
       found = TRUE;
-      g_list_delete_link (props->properties, current);
+      props->properties = g_list_delete_link (props->properties, current);
+      gst_props_entry_destroy (lentry);
     }
     else if (GST_PROPS_ENTRY_IS_VARIABLE (lentry)) {
       GST_PROPS_FLAG_UNSET (props, GST_PROPS_FIXED);
@@ -513,11 +1050,11 @@ void
 gst_props_debug (GstProps *props)
 {
   if (!props) {
-    GST_DEBUG (GST_CAT_PROPERTIES, "props (null)");
+    GST_CAT_DEBUG (GST_CAT_PROPERTIES, "props (null)");
     return;
   }
 	  
-  GST_DEBUG (GST_CAT_PROPERTIES, "props %p, refcount %d, flags %d", 
+  GST_CAT_DEBUG (GST_CAT_PROPERTIES, "props %p, refcount %d, flags %d", 
 		  props, props->refcount, props->flags);
 
   g_list_foreach (props->properties, (GFunc) gst_props_debug_entry, NULL);
@@ -630,14 +1167,24 @@ gst_props_add_to_int_list (GList *entries, GstPropsEntry *newentry)
   return g_list_prepend (entries, newentry);
 }
 
+GType
+gst_props_entry_get_type (void)
+{
+  return _gst_props_entry_type;
+}
+
+#define GST_PROPS_ENTRY_NEW(entry, name,var_args)				\
+G_STMT_START {									\
+  entry = gst_props_alloc_entry ();						\
+  entry->propid = g_quark_from_string (name);					\
+  GST_PROPS_ENTRY_FILL (entry, var_args);					\
+} G_STMT_END
 static GstPropsEntry*
 gst_props_entry_newv (const gchar *name, va_list var_args)
 {
   GstPropsEntry *entry;
 
-  entry = gst_props_alloc_entry ();
-  entry->propid = g_quark_from_string (name);
-  GST_PROPS_ENTRY_FILL (entry, var_args);
+  GST_PROPS_ENTRY_NEW (entry, name, var_args);
 
   return entry;
 }
@@ -677,21 +1224,7 @@ GstProps*
 gst_props_newv (const gchar *firstname, va_list var_args)
 {
   GstProps *props;
-  gboolean inlist = FALSE;
   const gchar *prop_name;
-  GstPropsEntry *list_entry = NULL;
-
-  typedef enum {
-      GST_PROPS_LIST_T_UNSET,
-      GST_PROPS_LIST_T_INTS,
-      GST_PROPS_LIST_T_FLOATS,
-      GST_PROPS_LIST_T_MISC,
-  } list_types;
-
-  /* type of the list */
-  list_types list_type = GST_PROPS_LIST_T_UNSET;
-  /* type of current item */
-  list_types entry_type = GST_PROPS_LIST_T_UNSET;
 
   if (firstname == NULL)
     return NULL;
@@ -704,79 +1237,10 @@ gst_props_newv (const gchar *firstname, va_list var_args)
   while (prop_name) {
     GstPropsEntry *entry;
 
-    entry = gst_props_alloc_entry ();
-    entry->propid = g_quark_from_string (prop_name);
-    GST_PROPS_ENTRY_FILL (entry, var_args);
-
-    switch (entry->propstype) {
-      case GST_PROPS_INT_TYPE:
-      case GST_PROPS_INT_RANGE_TYPE:
-	entry_type = GST_PROPS_LIST_T_INTS;
-	break;
-      case GST_PROPS_FLOAT_TYPE:
-      case GST_PROPS_FLOAT_RANGE_TYPE:
-	entry_type = GST_PROPS_LIST_T_FLOATS;
-	break;
-      case GST_PROPS_FOURCC_TYPE:
-      case GST_PROPS_BOOLEAN_TYPE:
-      case GST_PROPS_STRING_TYPE:
-	entry_type = GST_PROPS_LIST_T_MISC;
-	break;
-      case GST_PROPS_LIST_TYPE:
-	g_return_val_if_fail (inlist == FALSE, NULL);
-	inlist = TRUE;
-	list_entry = entry;
-	list_type = GST_PROPS_LIST_T_UNSET;
-	list_entry->data.list_data.entries = NULL;
-	break;
-      case GST_PROPS_END_TYPE:
-	g_return_val_if_fail (inlist == TRUE, NULL);
-
-	/* if list was of size 1, replace the list by a the item it contains */
-	if (g_list_length(list_entry->data.list_data.entries) == 1) {
-	  GstPropsEntry *subentry = (GstPropsEntry *)(list_entry->data.list_data.entries->data);
-	  list_entry->propstype = subentry->propstype;
-	  list_entry->data = subentry->data;
-          gst_props_entry_destroy (subentry);
-	}
-	else {
-	  list_entry->data.list_data.entries =
-		    g_list_reverse (list_entry->data.list_data.entries);
-	}
-
-        gst_props_entry_destroy (entry);
-	inlist = FALSE;
-	list_entry = NULL;
-        prop_name = va_arg (var_args, gchar*);
-	continue;
-      default:
-	g_warning ("unknown property type found %d for '%s'\n", entry->propstype, prop_name);
-        gst_props_entry_destroy (entry);
-	break;
-    }
-
-    if (inlist && (list_entry != entry)) {
-      if (list_type == GST_PROPS_LIST_T_UNSET) list_type = entry_type;
-      if (list_type != entry_type) {
-	g_warning ("property list contained incompatible entry types\n");
-      } else {
-	switch (list_type) {
-	  case GST_PROPS_LIST_T_INTS:
-	    list_entry->data.list_data.entries =
-		    gst_props_add_to_int_list (list_entry->data.list_data.entries, entry);
-	    break;
-	  default:
-	    list_entry->data.list_data.entries =
-		    g_list_prepend (list_entry->data.list_data.entries, entry);
-	    break;
-	}
-      }
-    }
-    else {
-      gst_props_add_entry (props, entry);
-    }
-    if (!inlist)
-      prop_name = va_arg (var_args, gchar*);
+    GST_PROPS_ENTRY_NEW (entry, prop_name, var_args);
+    gst_props_add_entry (props, entry);
+    
+    prop_name = va_arg (var_args, gchar*);
   }
 
   return props;
@@ -885,7 +1349,7 @@ gst_props_unref (GstProps *props)
 
   g_return_val_if_fail (props->refcount > 0, NULL);
 
-  GST_DEBUG (GST_CAT_PROPERTIES, "unref %p (%d->%d)", props, props->refcount, props->refcount-1);
+  GST_CAT_LOG (GST_CAT_PROPERTIES, "unref %p (%d->%d)", props, props->refcount, props->refcount-1);
   props->refcount--;
 
   if (props->refcount == 0) {
@@ -912,7 +1376,7 @@ gst_props_ref (GstProps *props)
 
   g_return_val_if_fail (props->refcount > 0, NULL);
 
-  GST_DEBUG (GST_CAT_PROPERTIES, "ref %p (%d->%d)", props, props->refcount, props->refcount+1);
+  GST_CAT_LOG (GST_CAT_PROPERTIES, "ref %p (%d->%d)", props, props->refcount, props->refcount+1);
 
   props->refcount++;
 
@@ -932,7 +1396,7 @@ gst_props_sink (GstProps *props)
   if (props == NULL)
     return;
 
-  GST_DEBUG (GST_CAT_PROPERTIES, "sink %p", props);
+  GST_CAT_LOG (GST_CAT_PROPERTIES, "sink %p", props);
 
   if (GST_PROPS_IS_FLOATING (props)) {
     GST_PROPS_FLAG_UNSET (props, GST_PROPS_FLOATING);
@@ -947,7 +1411,7 @@ gst_props_sink (GstProps *props)
  * Destroy the property, freeing all the memory that
  * was allocated.
  */
-void
+static void
 gst_props_destroy (GstProps *props)
 {
   if (props == NULL)
@@ -956,7 +1420,13 @@ gst_props_destroy (GstProps *props)
   g_list_foreach (props->properties, (GFunc) gst_props_entry_destroy, NULL);
   g_list_free (props->properties);
 
+#ifdef USE_POISONING
+  memset(props, 0xff, sizeof(*props));
+#endif
   gst_mem_chunk_free (_gst_props_chunk, props);
+#ifndef GST_DISABLE_TRACE
+  gst_alloc_trace_free (_props_trace, props);
+#endif
 }
 
 /**
@@ -969,7 +1439,7 @@ gst_props_destroy (GstProps *props)
  * given entry.
  */
 GstPropsEntry*
-gst_props_entry_copy (GstPropsEntry *entry)
+gst_props_entry_copy (const GstPropsEntry *entry)
 {
   GstPropsEntry *newentry;
 
@@ -1060,10 +1530,10 @@ gst_props_copy_on_write (GstProps *props)
  * @props: the props to query
  * @name: the name of the entry to get
  *
- * Get the props entry with the geven name
+ * Get the props entry with the given name
  *
- * Returns: The props entry with the geven name or NULL when
- * the entry was not found.
+ * Returns: The props entry with the given name or NULL if
+ * the entry does not exist.
  */
 const GstPropsEntry*
 gst_props_get_entry (GstProps *props, const gchar *name)
@@ -1071,9 +1541,10 @@ gst_props_get_entry (GstProps *props, const gchar *name)
   GList *lentry;
   GQuark quark;
 
-  g_return_val_if_fail (props != NULL, NULL);
   g_return_val_if_fail (name != NULL, NULL);
 
+  if (props == NULL) return FALSE;
+
   quark = g_quark_from_string (name);
 
   lentry = g_list_find_custom (props->properties, GINT_TO_POINTER (quark), props_find_func);
@@ -1154,7 +1625,7 @@ gst_props_has_fixed_property (GstProps *props, const gchar *name)
  * Returns: The type of the props entry.
  */
 GstPropsType
-gst_props_entry_get_type (const GstPropsEntry *entry)
+gst_props_entry_get_props_type (const GstPropsEntry *entry)
 {
   g_return_val_if_fail (entry != NULL, GST_PROPS_INVALID_TYPE);
 
@@ -1234,7 +1705,7 @@ gst_props_entry_get_safe (const GstPropsEntry *entry, ...)
   gboolean result;
   va_list var_args;
 
-  g_return_val_if_fail (entry != NULL, FALSE);
+  if(entry == NULL) return FALSE;
 
   va_start (var_args, entry);
   result = gst_props_entry_getv (entry, TRUE, var_args);
@@ -1489,7 +1960,7 @@ gst_props_entry_check_list_compatibility (GstPropsEntry *entry1, GstPropsEntry *
 static gboolean
 gst_props_entry_check_compatibility (GstPropsEntry *entry1, GstPropsEntry *entry2)
 {
-  GST_DEBUG (GST_CAT_PROPERTIES,"compare: %s %s", g_quark_to_string (entry1->propid), g_quark_to_string (entry2->propid));
+  GST_CAT_DEBUG (GST_CAT_PROPERTIES,"compare: %s %s", g_quark_to_string (entry1->propid), g_quark_to_string (entry2->propid));
 
   if (entry2->propstype == GST_PROPS_LIST_TYPE && entry1->propstype != GST_PROPS_LIST_TYPE) {
     return gst_props_entry_check_list_compatibility (entry1, entry2);
@@ -1535,7 +2006,7 @@ gst_props_entry_check_compatibility (GstPropsEntry *entry1, GstPropsEntry *entry
       switch (entry2->propstype) {
 	/* b   <--->   a */
         case GST_PROPS_FOURCC_TYPE:
-          GST_DEBUG(GST_CAT_PROPERTIES,"\"%c%c%c%c\" <--> \"%c%c%c%c\" ?",
+          GST_CAT_DEBUG (GST_CAT_PROPERTIES,"\"%c%c%c%c\" <--> \"%c%c%c%c\" ?",
 	    (entry2->data.fourcc_data>>0)&0xff,
 	    (entry2->data.fourcc_data>>8)&0xff,
 	    (entry2->data.fourcc_data>>16)&0xff,
@@ -1553,13 +2024,13 @@ gst_props_entry_check_compatibility (GstPropsEntry *entry1, GstPropsEntry *entry
       switch (entry2->propstype) {
 	/* b   <--->   a - d */
         case GST_PROPS_INT_RANGE_TYPE:
-          GST_DEBUG(GST_CAT_PROPERTIES,"%d <= %d <= %d ?",entry2->data.int_range_data.min,
+          GST_CAT_DEBUG (GST_CAT_PROPERTIES,"%d <= %d <= %d ?",entry2->data.int_range_data.min,
                     entry1->data.int_data,entry2->data.int_range_data.max);
 	  return (entry2->data.int_range_data.min <= entry1->data.int_data &&
 	          entry2->data.int_range_data.max >= entry1->data.int_data);
 	/* b   <--->   a */
         case GST_PROPS_INT_TYPE:
-          GST_DEBUG(GST_CAT_PROPERTIES,"%d == %d ?",entry1->data.int_data,entry2->data.int_data);
+          GST_CAT_DEBUG (GST_CAT_PROPERTIES,"%d == %d ?",entry1->data.int_data,entry2->data.int_data);
 	  return (entry2->data.int_data == entry1->data.int_data);
         default:
 	  break;
@@ -1590,7 +2061,7 @@ gst_props_entry_check_compatibility (GstPropsEntry *entry1, GstPropsEntry *entry
       switch (entry2->propstype) {
 	/* t   <--->   t */
         case GST_PROPS_STRING_TYPE:
-          GST_DEBUG(GST_CAT_PROPERTIES,"\"%s\" <--> \"%s\" ?",
+          GST_CAT_DEBUG (GST_CAT_PROPERTIES,"\"%s\" <--> \"%s\" ?",
 			  entry2->data.string_data.string, entry1->data.string_data.string);
           return (!strcmp (entry2->data.string_data.string, entry1->data.string_data.string));
         default:
@@ -1649,7 +2120,7 @@ gst_props_check_compatibility (GstProps *fromprops, GstProps *toprops)
 
     if (!gst_props_entry_check_compatibility (entry1, entry2)) {
 	compatible = FALSE; 
-	GST_DEBUG (GST_CAT_PROPERTIES, "%s are not compatible: ",
+	GST_CAT_DEBUG (GST_CAT_PROPERTIES, "%s are not compatible: ",
 		   g_quark_to_string (entry1->propid));
     }
 
@@ -1669,25 +2140,38 @@ end:
   return compatible;
 }
 
+static gint
+_props_entry_flexibility (GstPropsEntry *entry)
+{
+  gint rank;
+  switch (entry->propstype) {
+  default:
+    rank = 0;
+    break;
+  case GST_PROPS_INT_RANGE_TYPE:
+    rank = 1;
+    break;
+  case GST_PROPS_FLOAT_RANGE_TYPE:
+    rank = 2;
+    break;
+  case GST_PROPS_LIST_TYPE:
+    rank = 3;
+    break;
+  }
+  return rank;
+}
+
 static GstPropsEntry*
 gst_props_entry_intersect (GstPropsEntry *entry1, GstPropsEntry *entry2)
 {
   GstPropsEntry *result = NULL;
 
-  /* try to move the ranges and lists first */
-  switch (entry2->propstype) {
-    case GST_PROPS_INT_RANGE_TYPE:
-    case GST_PROPS_FLOAT_RANGE_TYPE:
-    case GST_PROPS_LIST_TYPE:
-    {
+  /* Swap more flexible types into entry1 */
+  if (_props_entry_flexibility (entry1) < _props_entry_flexibility (entry2)) {
       GstPropsEntry *temp;
-
       temp = entry1;
       entry1 = entry2;
       entry2 = temp;
-    }
-    default:
-      break;
   }
 
   switch (entry1->propstype) {
@@ -1885,6 +2369,15 @@ gst_props_entry_intersect (GstPropsEntry *entry1, GstPropsEntry *entry2)
       break;
   }
 
+  /* make caps debugging extremely verbose
+
+  GST_CAT_DEBUG (GST_CAT_CAPS, "intersecting %s: %s x %s => %s",
+		 g_quark_to_string (entry1->propid),
+		 gst_props_entry_to_string (entry1),
+		 gst_props_entry_to_string (entry2),
+		 result? gst_props_entry_to_string (result) : "fail");
+  */
+
   return result;
 }
 
@@ -2096,17 +2589,17 @@ gst_props_save_thyself_func (GstPropsEntry *entry, xmlNodePtr parent)
     case GST_PROPS_FLOAT_TYPE:
       subtree = xmlNewChild (parent, NULL, "float", NULL);
       xmlNewProp (subtree, "name", g_quark_to_string (entry->propid));
-      str = g_strdup_printf ("%f", entry->data.float_data);
+      str = g_strdup_printf ("%g", entry->data.float_data);
       xmlNewProp (subtree, "value", str);
       g_free(str);
       break;
     case GST_PROPS_FLOAT_RANGE_TYPE:
       subtree = xmlNewChild (parent, NULL, "floatrange", NULL);
       xmlNewProp (subtree, "name", g_quark_to_string (entry->propid));
-      str = g_strdup_printf ("%f", entry->data.float_range_data.min);
+      str = g_strdup_printf ("%g", entry->data.float_range_data.min);
       xmlNewProp (subtree, "min", str);
       g_free(str);
-      str = g_strdup_printf ("%f", entry->data.float_range_data.max);
+      str = g_strdup_printf ("%g", entry->data.float_range_data.max);
       xmlNewProp (subtree, "max", str);
       g_free(str);
       break;
@@ -2318,4 +2811,3 @@ gst_props_load_thyself (xmlNodePtr parent)
   return props;
 }
 #endif /* GST_DISABLE_LOADSAVE_REGISTRY */
-
