@@ -20,7 +20,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* #define DEBUG_ENABLED */
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <stdio.h>
 #include <errno.h>
 #include <sys/types.h>
@@ -31,10 +34,11 @@
 
 #include <gst/gst_private.h>
 #include <gst/gstelement.h>
-#include <gst/gsttype.h>
+#include <gst/gsttypefind.h>
 #include <gst/gstscheduler.h>
 #include <gst/gstautoplug.h>
 #include <gst/gsturi.h>
+#include <gst/gstinfo.h>
 
 #include "gstxmlregistry.h"
 
@@ -394,8 +398,8 @@ gst_xml_registry_add_path_list_func (GstXMLRegistry *registry)
     g_markup_parse_context_parse (context, text, size, &error);
 
     if (error) {
-      fprintf (stderr, "ERROR: parsing registry %s: %s\n", 
-	       registry->location, error->message);
+      GST_ERROR ("parsing registry %s: %s\n", 
+	         registry->location, error->message);
       g_free (text);
       fclose (reg);
       return;
@@ -422,7 +426,7 @@ plugin_times_older_than_recurse(gchar *path, time_t regtime)
   time_t pathtime = get_time(path);
 
   if (pathtime > regtime) {
-    GST_INFO (GST_CAT_PLUGIN_LOADING,
+    GST_CAT_INFO (GST_CAT_PLUGIN_LOADING,
                      "time for %s was %ld; more recent than registry time of %ld\n",
                      path, (long)pathtime, (long)regtime);
     return FALSE;
@@ -455,7 +459,7 @@ plugin_times_older_than(GList *paths, time_t regtime)
    */
 
   while (paths) {
-    GST_DEBUG (GST_CAT_PLUGIN_LOADING,
+    GST_CAT_DEBUG (GST_CAT_PLUGIN_LOADING,
                       "comparing plugin times from %s with %ld\n",
                       (gchar *)paths->data, (long) regtime);
     if(!plugin_times_older_than_recurse(paths->data, regtime))
@@ -485,10 +489,10 @@ gst_xml_registry_open_func (GstXMLRegistry *registry, GstXMLRegistryMode mode)
       /* if it's not writable, then don't bother */
       if (!(gst_registry->flags & GST_REGISTRY_WRITABLE))
       {
-        GST_INFO (GST_CAT_GST_INIT, "Registry isn't writable");
+        GST_CAT_INFO (GST_CAT_GST_INIT, "Registry isn't writable");
 	return FALSE;
       }
-      GST_INFO (GST_CAT_GST_INIT, "Registry doesn't exist, trying to build...");
+      GST_CAT_INFO (GST_CAT_GST_INIT, "Registry doesn't exist, trying to build...");
       gst_registry_rebuild (gst_registry);
       gst_registry_save (gst_registry);
       /* FIXME: verify that the flags actually get updated ! */
@@ -502,18 +506,18 @@ gst_xml_registry_open_func (GstXMLRegistry *registry, GstXMLRegistryMode mode)
 
     if (!plugin_times_older_than (paths, get_time (registry->location))) {
       if (gst_registry->flags & GST_REGISTRY_WRITABLE) {
-        GST_INFO (GST_CAT_GST_INIT, "Registry out of date, rebuilding...");
+        GST_CAT_INFO (GST_CAT_GST_INIT, "Registry out of date, rebuilding...");
       
         gst_registry_rebuild (gst_registry);
 
         gst_registry_save (gst_registry);
 
         if (!plugin_times_older_than (paths, get_time (registry->location))) {
-          GST_INFO (GST_CAT_GST_INIT, "Registry still out of date, something is wrong...");
+          GST_CAT_INFO (GST_CAT_GST_INIT, "Registry still out of date, something is wrong...");
           return FALSE;
         }
       } else {
-        GST_INFO (GST_CAT_PLUGIN_LOADING, "Can't write to this registry and it's out of date, ignoring it");
+        GST_CAT_INFO (GST_CAT_PLUGIN_LOADING, "Can't write to this registry and it's out of date, ignoring it");
         return FALSE;
       }
     }
@@ -596,7 +600,7 @@ gst_xml_registry_load (GstRegistry *registry)
     g_markup_parse_context_parse (xmlregistry->context, text, size, &error);
 
     if (error) {
-      fprintf(stderr, "ERROR: parsing registry: %s\n", error->message);
+      GST_ERROR ("parsing registry: %s\n", error->message);
       g_free (text);
       CLASS (xmlregistry)->close_func (xmlregistry);
       return FALSE;
@@ -613,7 +617,7 @@ gst_xml_registry_load (GstRegistry *registry)
   seconds = g_timer_elapsed (timer, NULL);
   g_timer_destroy (timer);
 
-  GST_INFO (0, "registry: loaded %s in %f seconds\n          (%s)", 
+  GST_INFO ( "registry: loaded %s in %f seconds\n          (%s)", 
 	   registry->name, seconds, xmlregistry->location);
 
   CLASS (xmlregistry)->close_func (xmlregistry);
@@ -626,13 +630,17 @@ static GstRegistryReturn
 gst_xml_registry_load_plugin (GstRegistry *registry, GstPlugin *plugin)
 {
   GError *error = NULL;
+  GstPlugin *loaded_plugin;
 
   /* FIXME: add gerror support */
-  if (!gst_plugin_load_plugin (plugin, &error)) {
+  loaded_plugin = gst_plugin_load_file (plugin->filename, &error);
+  if (!plugin) {
     if (error) {
-      g_warning ("could not load plugin %s: %s", plugin->name, error->message);
+      g_warning ("could not load plugin %s: %s", plugin->desc.name, error->message);
     }
     return GST_REGISTRY_PLUGIN_LOAD_ERROR;
+  } else if (loaded_plugin != plugin) {
+    g_critical ("how to remove plugins?");
   }
 
   return GST_REGISTRY_OK;
@@ -645,14 +653,29 @@ gst_xml_registry_parse_plugin (GMarkupParseContext *context, const gchar *tag, c
   GstPlugin *plugin = registry->current_plugin;
   
   if (!strcmp (tag, "name")) {
-    plugin->name = g_strndup (text, text_len);
+    plugin->desc.name = g_strndup (text, text_len);
   }
-  else if (!strcmp (tag, "longname")) {
-    plugin->longname = g_strndup (text, text_len);
+  else if (!strcmp (tag, "description")) {
+    plugin->desc.description = g_strndup (text, text_len);
   }
   else if (!strcmp (tag, "filename")) {
     plugin->filename = g_strndup (text, text_len);
   }
+  else if (!strcmp (tag, "version")) {
+    plugin->desc.version = g_strndup (text, text_len);
+  }
+  else if (!strcmp (tag, "copyright")) {
+    plugin->desc.copyright = g_strndup (text, text_len);
+  }
+  else if (!strcmp (tag, "license")) {
+    plugin->desc.license = g_strndup (text, text_len);
+  }
+  else if (!strcmp (tag, "package")) {
+    plugin->desc.package = g_strndup (text, text_len);
+  }
+  else if (!strcmp (tag, "origin")) {
+    plugin->desc.origin = g_strndup (text, text_len);
+  }
   
   return TRUE;
 }
@@ -664,74 +687,74 @@ gst_xml_registry_parse_element_factory (GMarkupParseContext *context, const gcha
   GstElementFactory *factory = GST_ELEMENT_FACTORY (registry->current_feature);
 
   if (!strcmp (tag, "name")) {
+    g_free (registry->current_feature->name);
     registry->current_feature->name = g_strndup (text, text_len);
   }
   else if (!strcmp (tag, "longname")) {
-    factory->details->longname = g_strndup (text, text_len);
+    g_free (factory->details.longname);
+    factory->details.longname = g_strndup (text, text_len);
   }
   else if (!strcmp(tag, "class")) {
-    factory->details->klass = g_strndup (text, text_len);
+    g_free (factory->details.klass);
+    factory->details.klass = g_strndup (text, text_len);
   }
   else if (!strcmp(tag, "description")) {
-    factory->details->description = g_strndup (text, text_len);
-  }
-  else if (!strcmp(tag, "version")) {
-    factory->details->version = g_strndup (text, text_len);
+    g_free (factory->details.description);
+    factory->details.description = g_strndup (text, text_len);
   }
   else if (!strcmp(tag, "author")) {
-    factory->details->author = g_strndup (text, text_len);
-  }
-  else if (!strcmp(tag, "copyright")) {
-    factory->details->copyright = g_strndup (text, text_len);
+    g_free (factory->details.author);
+    factory->details.author = g_strndup (text, text_len);
   }
   else if (!strcmp(tag, "rank")) {
-    gint value;
-    sscanf (text, "%d", &value);
-    factory->rank = value;
+    gint rank;
+    gchar *ret;
+     rank = strtol (text, &ret, 0);
+    if (ret == text + text_len) {
+     gst_plugin_feature_set_rank (GST_PLUGIN_FEATURE (factory), rank);
+    }
   }
-
+  
   return TRUE;
 }
 
-static GstCaps*
-gst_type_type_find_dummy (GstBuffer *buffer, gpointer priv)
-{
-  GstTypeFactory *factory = (GstTypeFactory *)priv;
-
-  GST_DEBUG (GST_CAT_TYPES,"gsttype: need to load typefind function for %s", factory->mime);
-
-  if (gst_plugin_feature_ensure_loaded (GST_PLUGIN_FEATURE (factory))) {
-    if (factory->typefindfunc)
-    {
-      if (factory->typefindfunc == gst_type_type_find_dummy) {
-        g_warning ("factory->typefindfunc set to _dummy, not recursing, FIXME");
-      }
-      else
-      {
-        GstCaps *res = factory->typefindfunc (buffer, factory);
-        if (res)
-          return res;
-      }
-    }
-  }
-  return NULL;
-}
-
-
 static gboolean
-gst_xml_registry_parse_type_factory (GMarkupParseContext *context, const gchar *tag, const gchar *text,
-                                     gsize text_len, GstXMLRegistry *registry, GError **error)
+gst_xml_registry_parse_type_find_factory (GMarkupParseContext *context, const gchar *tag, const gchar *text,
+					  gsize text_len, GstXMLRegistry *registry, GError **error)
 {
-  GstTypeFactory *factory = GST_TYPE_FACTORY (registry->current_feature);
+  GstTypeFindFactory *factory = GST_TYPE_FIND_FACTORY (registry->current_feature);
 
   if (!strcmp (tag, "name")) {
     registry->current_feature->name = g_strndup (text, text_len);
   }
-  else if (!strcmp (tag, "mime")) {
-    factory->mime = g_strndup (text, text_len);
+  else if (!strcmp(tag, "rank")) {
+    glong rank;
+    gchar *ret;
+     rank = strtol (text, &ret, 0);
+    if (ret == text + text_len) {
+     gst_plugin_feature_set_rank (GST_PLUGIN_FEATURE (factory), rank);
+    }
   }
-  else if (!strcmp(tag, "extensions")) {
-    factory->exts = g_strndup (text, text_len);
+  /* FIXME!!
+  else if (!strcmp (tag, "caps")) {
+    factory->caps = g_strndup (text, text_len);
+  }*/
+  else if (!strcmp(tag, "extension")) {
+    gchar **new;
+    gchar **old = factory->extensions;
+    gint i = 0;
+    
+    /* expensive, but cycles are cheap... */
+    if (old)
+      while (old[i]) i++;
+    new = g_new0 (gchar *, i + 2);
+    new[i] = g_strndup (text, text_len);
+    while (i > 0) {
+      i--;
+      new[i] = old[i];
+    }
+    g_free (old);
+    factory->extensions = new;
   }
 
   return TRUE;
@@ -906,15 +929,12 @@ gst_xml_registry_start_element (GMarkupParseContext *context,
 	  if (GST_IS_ELEMENT_FACTORY (feature)) {
 	    GstElementFactory *factory = GST_ELEMENT_FACTORY (feature);
 
-	    factory->details_dynamic = TRUE;
-	    factory->details = g_new0(GstElementDetails, 1);
 	    factory->padtemplates = NULL;
-	    factory->rank = 0;
 	    xmlregistry->parser = gst_xml_registry_parse_element_factory;
 	    break;
 	  }
-	  else if (GST_IS_TYPE_FACTORY (feature)) {
-	    xmlregistry->parser = gst_xml_registry_parse_type_factory;
+	  else if (GST_IS_TYPE_FIND_FACTORY (feature)) {
+	    xmlregistry->parser = gst_xml_registry_parse_type_find_factory;
 	  }
 	  else if (GST_IS_SCHEDULER_FACTORY (feature)) {
 	    xmlregistry->parser = gst_xml_registry_parse_scheduler_factory;
@@ -1095,20 +1115,11 @@ gst_xml_registry_end_element (GMarkupParseContext *context,
       break;
     case GST_XML_REGISTRY_FEATURE:
       if (!strcmp (element_name, "feature")) {
-	if (GST_IS_TYPE_FACTORY (xmlregistry->current_feature)) {
-          GstTypeFactory *factory = GST_TYPE_FACTORY (xmlregistry->current_feature);
-	  gst_type_register (factory);
-	}
 	xmlregistry->state = GST_XML_REGISTRY_PLUGIN;
 	xmlregistry->parser = gst_xml_registry_parse_plugin;
 	gst_plugin_add_feature (xmlregistry->current_plugin, xmlregistry->current_feature);
 	xmlregistry->current_feature = NULL;
       }
-      else if (!strcmp (element_name, "typefind")) {
-        GstTypeFactory *factory = GST_TYPE_FACTORY (xmlregistry->current_feature);
-
-        factory->typefindfunc = gst_type_type_find_dummy;
-      }
       break;
     case GST_XML_REGISTRY_PADTEMPLATE:
       if (!strcmp (element_name, "padtemplate")) {
@@ -1123,7 +1134,7 @@ gst_xml_registry_end_element (GMarkupParseContext *context,
 	xmlregistry->name_template = NULL;
 	xmlregistry->caps = NULL;
 
-	gst_element_factory_add_pad_template (GST_ELEMENT_FACTORY (xmlregistry->current_feature),
+	__gst_element_factory_add_pad_template (GST_ELEMENT_FACTORY (xmlregistry->current_feature),
 					      template);
 	xmlregistry->state = GST_XML_REGISTRY_FEATURE;
 	xmlregistry->parser = gst_xml_registry_parse_element_factory;
@@ -1206,7 +1217,7 @@ static void
 gst_xml_registry_error (GMarkupParseContext *context, GError *error,
                         gpointer user_data)
 {
-  g_print ("error %s\n", error->message);
+  GST_ERROR ("%s\n", error->message);
 }
 
 static void
@@ -1278,12 +1289,19 @@ gst_xml_registry_paths_text (GMarkupParseContext *context, const gchar *text,
 #define PUT_ESCAPED(tag,value) 					\
 G_STMT_START{ 							\
   const gchar *toconv = value;					\
-  if (value) {							\
+  if (toconv) {							\
     gchar *v = g_markup_escape_text (toconv, strlen (toconv));	\
     CLASS (xmlregistry)->save_func (xmlregistry, "<%s>%s</%s>\n", tag, v, tag);			\
     g_free (v);							\
   }								\
 }G_STMT_END
+#define PUT_ESCAPED_INT(tag,value)				\
+G_STMT_START{ 							\
+  gchar *save = g_strdup_printf ("%ld", (glong) value);		\
+  CLASS (xmlregistry)->save_func (xmlregistry, "<%s>%s</%s>\n", tag, save, tag);		\
+  g_free (save);      						\
+}G_STMT_END
+
 
 static gboolean
 gst_xml_registry_save_props_func (GstPropsEntry *entry, 
@@ -1293,7 +1311,7 @@ gst_xml_registry_save_props_func (GstPropsEntry *entry,
 
   name = gst_props_entry_get_name (entry);
 
-  switch (gst_props_entry_get_type (entry)) {
+  switch (gst_props_entry_get_props_type (entry)) {
     case GST_PROPS_INT_TYPE:
     {
       gint value;
@@ -1326,7 +1344,8 @@ gst_xml_registry_save_props_func (GstPropsEntry *entry,
     {
       guint32 fourcc;
       gst_props_entry_get_fourcc_int (entry, &fourcc);
-      CLASS (xmlregistry)->save_func (xmlregistry, "<!--%4.4s-->\n", (gchar *)&fourcc);
+      CLASS (xmlregistry)->save_func (xmlregistry, "<!-- "GST_FOURCC_FORMAT" -->\n", 
+				      GST_FOURCC_ARGS (fourcc));
       CLASS (xmlregistry)->save_func (xmlregistry, "<fourcc name=\"%s\" hexvalue=\"%08x\"/>\n", name, fourcc);
       break;
     }
@@ -1345,7 +1364,7 @@ gst_xml_registry_save_props_func (GstPropsEntry *entry,
       break;
     }
     default:
-      g_warning ("trying to save unknown property type %d", gst_props_entry_get_type (entry));
+      g_warning ("trying to save unknown property type %d", gst_props_entry_get_props_type (entry));
       return FALSE;
   }
   return TRUE;
@@ -1361,7 +1380,7 @@ gst_xml_registry_save_props (GstXMLRegistry *xmlregistry, GstProps *props)
   while (proplist) {
     GstPropsEntry *entry = (GstPropsEntry *) proplist->data;
 
-    switch (gst_props_entry_get_type (entry)) {
+    switch (gst_props_entry_get_props_type (entry)) {
       case GST_PROPS_LIST_TYPE: 
       {
 	const GList *list;
@@ -1388,7 +1407,7 @@ gst_xml_registry_save_caps (GstXMLRegistry *xmlregistry, GstCaps *caps)
   while (caps) {
     CLASS (xmlregistry)->save_func (xmlregistry, "<capscomp>\n");
     PUT_ESCAPED ("name", caps->name);
-    PUT_ESCAPED ("type", gst_type_find_by_id (caps->id)->mime);
+    PUT_ESCAPED ("type", gst_caps_get_mime (caps));
 
     if (caps->properties) {
       CLASS (xmlregistry)->save_func (xmlregistry, "<properties>\n");
@@ -1438,22 +1457,19 @@ gst_xml_registry_save_feature (GstXMLRegistry *xmlregistry, GstPluginFeature *fe
 {
   PUT_ESCAPED ("name", feature->name);
 
+  if (feature->rank > 0) {
+    gint rank = feature->rank;
+    CLASS (xmlregistry)->save_func (xmlregistry, "<rank>%d</rank>\n", rank);
+  }	
+    
   if (GST_IS_ELEMENT_FACTORY (feature)) {
     GstElementFactory *factory = GST_ELEMENT_FACTORY (feature);
     GList *templates;
 
-    PUT_ESCAPED ("longname", factory->details->longname);
-    PUT_ESCAPED ("class", factory->details->klass);
-    PUT_ESCAPED ("description", factory->details->description);
-    PUT_ESCAPED ("version", factory->details->version);
-    PUT_ESCAPED ("author", factory->details->author);
-    PUT_ESCAPED ("copyright", factory->details->copyright);
-    
-    /* only write the rank if it is greater than zero */
-    if (factory->rank > 0) {
-      gint rank = factory->rank;
-      CLASS (xmlregistry)->save_func (xmlregistry, "<rank>%d</rank>\n", rank);
-    }	
+    PUT_ESCAPED ("longname", factory->details.longname);
+    PUT_ESCAPED ("class", factory->details.klass);
+    PUT_ESCAPED ("description", factory->details.description);
+    PUT_ESCAPED ("author", factory->details.author);
     
     templates = factory->padtemplates;
 
@@ -1467,13 +1483,20 @@ gst_xml_registry_save_feature (GstXMLRegistry *xmlregistry, GstPluginFeature *fe
       templates = g_list_next (templates);
     }
   }
-  else if (GST_IS_TYPE_FACTORY (feature)) {
-    GstTypeFactory *factory = GST_TYPE_FACTORY (feature);
-
-    PUT_ESCAPED ("mime", factory->mime);
-    PUT_ESCAPED ("extensions", factory->exts);
-    if (factory->typefindfunc) {
-      CLASS (xmlregistry)->save_func (xmlregistry, "<typefind/>\n");
+  else if (GST_IS_TYPE_FIND_FACTORY (feature)) {
+    GstTypeFindFactory *factory = GST_TYPE_FIND_FACTORY (feature);
+    gint i = 0;
+    /* FIXME 
+    if (factory->caps) {
+      CLASS (xmlregistry)->save_func (xmlregistry, "<caps>\n");
+      gst_xml_registry_save_caps (xmlregistry, factory->caps);
+      CLASS (xmlregistry)->save_func (xmlregistry, "</caps>\n");
+    } */
+    if (factory->extensions) {
+      while (factory->extensions[i]) {
+        PUT_ESCAPED ("extension", factory->extensions[i]);
+        i++;
+      }
     }
   }
   else if (GST_IS_SCHEDULER_FACTORY (feature)) {
@@ -1502,9 +1525,14 @@ gst_xml_registry_save_plugin (GstXMLRegistry *xmlregistry, GstPlugin *plugin)
 {
   GList *walk;
   
-  PUT_ESCAPED ("name", plugin->name);
-  PUT_ESCAPED ("longname", plugin->longname);
+  PUT_ESCAPED ("name", plugin->desc.name);
+  PUT_ESCAPED ("description", plugin->desc.description);
   PUT_ESCAPED ("filename", plugin->filename);
+  PUT_ESCAPED ("version", plugin->desc.version);
+  PUT_ESCAPED ("license", plugin->desc.license);
+  PUT_ESCAPED ("copyright", plugin->desc.copyright);
+  PUT_ESCAPED ("package", plugin->desc.package);
+  PUT_ESCAPED ("origin", plugin->desc.origin);
 
   walk = plugin->features;
 
@@ -1573,6 +1601,7 @@ gst_xml_registry_rebuild_recurse (GstXMLRegistry *registry,
                                   const gchar *directory)
 {
   GDir *dir;
+  gchar *temp;
   GList *ret = NULL;
 
   dir = g_dir_open (directory, 0, NULL);
@@ -1594,13 +1623,9 @@ gst_xml_registry_rebuild_recurse (GstXMLRegistry *registry,
     }
     g_dir_close (dir);
   } else {
-    if (strstr (directory, ".so")) {
-      gchar *temp;
-
-      if ((temp = strstr (directory, ".so")) &&
-          (!strcmp (temp, ".so"))) {
-        ret = g_list_prepend (ret, gst_plugin_new (directory));
-      }
+    if ((temp = strstr (directory, G_MODULE_SUFFIX)) &&
+        (!strcmp (temp, G_MODULE_SUFFIX))) {
+      ret = g_list_prepend (ret, g_strdup (directory));
     }
   }
 
@@ -1612,7 +1637,8 @@ gst_xml_registry_rebuild (GstRegistry *registry)
 {
   GList *walk = NULL, *plugins = NULL, *prune = NULL;
   GError *error = NULL;
-  gint length;
+  guint length;
+  GstPlugin *plugin;
   GstXMLRegistry *xmlregistry = GST_XML_REGISTRY (registry);
  
   walk = registry->paths;
@@ -1620,7 +1646,7 @@ gst_xml_registry_rebuild (GstRegistry *registry)
   while (walk) {
     gchar *path = (gchar *) walk->data;
     
-    GST_INFO (GST_CAT_PLUGIN_LOADING, 
+    GST_CAT_INFO (GST_CAT_PLUGIN_LOADING, 
 	      "Rebuilding registry %p in directory %s...", registry, path);
 
     plugins = g_list_concat (plugins, 
@@ -1638,9 +1664,10 @@ gst_xml_registry_rebuild (GstRegistry *registry)
     walk = plugins;
     while (walk) {
       g_assert (walk->data);
-      if (gst_plugin_load_plugin (GST_PLUGIN (walk->data), NULL)) {
+      plugin = gst_plugin_load_file ((gchar *) walk->data, NULL);
+      if (plugin) {
         prune = g_list_prepend (prune, walk->data);
-        gst_registry_add_plugin (registry, GST_PLUGIN (walk->data));
+        gst_registry_add_plugin (registry, plugin);
       }
 
       walk = g_list_next (walk);
@@ -1649,6 +1676,7 @@ gst_xml_registry_rebuild (GstRegistry *registry)
     walk = prune;
     while (walk) {
       plugins = g_list_remove (plugins, walk->data);
+      g_free (walk->data);
       walk = g_list_next (walk);
     }
     g_list_free (prune);
@@ -1657,16 +1685,14 @@ gst_xml_registry_rebuild (GstRegistry *registry)
   
   walk = plugins;
   while (walk) {
-    if (gst_plugin_load_plugin (GST_PLUGIN (walk->data), &error)) {
+    if ((plugin = gst_plugin_load_file ((gchar *) walk->data, &error))) {
       g_warning ("Bizarre behavior: plugin %s actually loaded", 
-	         ((GstPlugin *) walk->data)->filename);
+	         (gchar *) walk->data);
+      gst_registry_add_plugin (registry, plugin);
     } else {
-      GST_INFO (GST_CAT_PLUGIN_LOADING, "Plugin %s failed to load: %s", 
-                ((GstPlugin *) walk->data)->filename, error->message);
-      g_print ("Plugin %s failed to load\n",
-                ((GstPlugin *) walk->data)->filename);
+      GST_CAT_INFO (GST_CAT_PLUGIN_LOADING, "Plugin %s failed to load: %s", 
+                (gchar *) walk->data, error->message);
 
-      g_free (((GstPlugin *) walk->data)->filename);
       g_free (walk->data);
       g_error_free (error);
       error = NULL;
