@@ -1,9 +1,13 @@
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <gst/gst.h>
 #include <gst/control/control.h>
 #include <string.h>
 
-static void 
-print_prop (GstPropsEntry *prop, gboolean showname, const gchar *pfx) 
+static void
+print_prop (GstPropsEntry *prop, gboolean showname, const gchar *pfx)
 {
   GstPropsType type;
 
@@ -12,7 +16,7 @@ print_prop (GstPropsEntry *prop, gboolean showname, const gchar *pfx)
   else
     g_print(pfx);
 
-  type = gst_props_entry_get_type (prop);
+  type = gst_props_entry_get_props_type (prop);
 
   switch (type) {
     case GST_PROPS_INT_TYPE:
@@ -33,14 +37,14 @@ print_prop (GstPropsEntry *prop, gboolean showname, const gchar *pfx)
     {
       gfloat val;
       gst_props_entry_get_float (prop, &val);
-      g_print("Float: %f\n", val);
+      g_print("Float: %g\n", val);
       break;
     }
     case GST_PROPS_FLOAT_RANGE_TYPE:
     {
       gfloat min, max;
       gst_props_entry_get_float_range (prop, &min, &max);
-      g_print("Float range: %f - %f\n", min, max);
+      g_print("Float range: %g - %g\n", min, max);
       break;
     }
     case GST_PROPS_BOOLEAN_TYPE:
@@ -62,10 +66,10 @@ print_prop (GstPropsEntry *prop, gboolean showname, const gchar *pfx)
       guint32 val;
       gst_props_entry_get_fourcc_int (prop, &val);
       g_print("FourCC: '%c%c%c%c'\n",
-             (gchar)( val        & 0xff), 
-	     (gchar)((val >> 8)  & 0xff),
-             (gchar)((val >> 16) & 0xff), 
-	     (gchar)((val >> 24) & 0xff));
+             (gchar)( val        & 0xff),
+             (gchar)((val >> 8)  & 0xff),
+             (gchar)((val >> 16) & 0xff),
+             (gchar)((val >> 24) & 0xff));
       break;
     }
     case GST_PROPS_LIST_TYPE:
@@ -92,8 +96,8 @@ print_prop (GstPropsEntry *prop, gboolean showname, const gchar *pfx)
   }
 }
 
-static void 
-print_props (GstProps *properties, const gchar *pfx) 
+static void
+print_props (GstProps *properties, const gchar *pfx)
 {
   GList *props;
   GstPropsEntry *prop;
@@ -107,19 +111,12 @@ print_props (GstProps *properties, const gchar *pfx)
   }
 }
 
-static void 
-print_caps (const GstCaps *caps, const gchar *pfx) 
+static void
+print_caps (GstCaps *caps, const gchar *pfx)
 {
   while (caps) {
-    GstType *type;
-
     g_print ("%s'%s': (%sfixed)\n", pfx, caps->name, (GST_CAPS_IS_FIXED (caps) ? "" : "NOT "));
-
-    type = gst_type_find_by_id (caps->id);
-    if (type) 
-      g_print ("%s  MIME type: '%s':\n", pfx, type->mime);
-    else
-      g_print ("%s  MIME type: 'unknown/unknown':\n", pfx);
+    g_print ("%s  MIME type: '%s':\n", pfx, gst_caps_get_mime (caps));
 
     if (caps->properties) {
       gchar *prefix = g_strdup_printf ("%s  ", pfx);
@@ -133,8 +130,8 @@ print_caps (const GstCaps *caps, const gchar *pfx)
   }
 }
 
-static void 
-print_formats (const GstFormat *formats) 
+static void
+print_formats (const GstFormat *formats)
 {
   while (formats && *formats) {
     const GstFormatDefinition *definition;
@@ -142,7 +139,7 @@ print_formats (const GstFormat *formats)
     definition = gst_format_get_details (*formats);
     if (definition)
       g_print ("\t\t(%d):\t%s (%s)\n", *formats,
-	       definition->nick, definition->description);
+               definition->nick, definition->description);
     else
       g_print ("\t\t(%d):\tUnknown format\n", *formats);
 
@@ -150,8 +147,8 @@ print_formats (const GstFormat *formats)
   }
 }
 
-static void 
-print_query_types (const GstQueryType *types) 
+static void
+print_query_types (const GstQueryType *types)
 {
   while (types && *types) {
     const GstQueryTypeDefinition *definition;
@@ -159,7 +156,7 @@ print_query_types (const GstQueryType *types)
     definition = gst_query_type_get_details (*types);
     if (definition)
       g_print ("\t\t(%d):\t%s (%s)\n", *types,
-	       definition->nick, definition->description);
+               definition->nick, definition->description);
     else
       g_print ("\t\t(%d):\tUnknown query format\n", *types);
 
@@ -167,9 +164,10 @@ print_query_types (const GstQueryType *types)
   }
 }
 
-static void 
-print_event_masks (const GstEventMask *masks) 
+static void
+print_event_masks (const GstEventMask *masks)
 {
+#ifndef GST_DISABLE_ENUMTYPES
   GType event_type;
   GEnumClass *klass;
   GType event_flags;
@@ -185,13 +183,13 @@ print_event_masks (const GstEventMask *masks)
     switch (masks->type) {
       case GST_EVENT_SEEK:
         flags = masks->flags;
-	event_flags = gst_seek_type_get_type ();
-  	flags_class = (GFlagsClass *) g_type_class_ref (event_flags);
+        event_flags = gst_seek_type_get_type ();
+        flags_class = (GFlagsClass *) g_type_class_ref (event_flags);
         break;
       default:
         break;
     }
-    
+
     value = g_enum_get_value (klass, masks->type);
     g_print ("\t\t%s ", value->value_nick);
 
@@ -201,18 +199,19 @@ print_event_masks (const GstEventMask *masks)
       if (flags & 1) {
         value = g_flags_get_first_value (flags_class, 1 << index);
 
-	if (value)
+        if (value)
           g_print ("| %s ", value->value_nick);
-	else
+        else
           g_print ("| ? ");
       }
       flags >>= 1;
       index++;
     }
     g_print ("\n");
-    
+
     masks++;
   }
+#endif
 }
 
 static void
@@ -228,29 +227,29 @@ output_hierarchy (GType type, gint level, gint *maxlevel)
 
   if (parent)
     output_hierarchy (parent, level, maxlevel);
-  
+
   for (i=1; i<*maxlevel-level; i++)
    g_print ("      ");
   if (*maxlevel-level)
     g_print (" +----");
 
   g_print ("%s\n", g_type_name (type));
-	  
+
   if (level == 1)
     g_print ("\n");
 }
 
 static void
-print_element_properties (GstElement *element) 
+print_element_properties (GstElement *element)
 {
   GParamSpec **property_specs;
   gint num_properties,i;
   gboolean readable;
-  
+  const char *string_val;
 
-  property_specs = g_object_class_list_properties 
-	             (G_OBJECT_GET_CLASS (element), &num_properties);
-  g_print("\nElement Arguments:\n");
+  property_specs = g_object_class_list_properties
+                     (G_OBJECT_GET_CLASS (element), &num_properties);
+  g_print("\nElement Properties:\n");
 
   for (i = 0; i < num_properties; i++) {
     GValue value = { 0, };
@@ -264,176 +263,201 @@ print_element_properties (GstElement *element)
     }
 
     g_print("  %-20s: %s\n", g_param_spec_get_name (param),
-		               g_param_spec_get_blurb (param));
+                               g_param_spec_get_blurb (param));
 
     switch (G_VALUE_TYPE (&value)) {
-      case G_TYPE_STRING: 
-	g_print ("%-23.23s String. ", "");
-	if (readable) g_print ("(Default \"%s\")", g_value_get_string (&value));
-	break;
-      case G_TYPE_BOOLEAN: 
-	g_print ("%-23.23s Boolean. ", "");
-	if (readable) g_print ("(Default %s)", (g_value_get_boolean (&value) ? "true" : "false"));
-	break;
-      case G_TYPE_ULONG: 
+      case G_TYPE_STRING:
+        string_val = g_value_get_string (&value);
+        g_print ("%-23.23s String. ", "");
+        if (readable) {
+          if (string_val == NULL)
+            g_print ("(Default \"\")");
+          else
+            g_print ("(Default \"%s\")", g_value_get_string (&value));
+        }
+        break;
+      case G_TYPE_BOOLEAN:
+        g_print ("%-23.23s Boolean. ", "");
+        if (readable) g_print ("(Default %s)", (g_value_get_boolean (&value) ? "true" : "false"));
+        break;
+      case G_TYPE_ULONG:
       {
-	GParamSpecULong *pulong = G_PARAM_SPEC_ULONG (param);
-	g_print("%-23.23s Unsigned Long. ", ""); 
-	if (readable) g_print("Range: %lu - %lu (Default %lu)",  
-			pulong->minimum, pulong->maximum, g_value_get_ulong (&value));
-	break;
+        GParamSpecULong *pulong = G_PARAM_SPEC_ULONG (param);
+        g_print("%-23.23s Unsigned Long. ", "");
+        if (readable) g_print("Range: %lu - %lu (Default %lu)",
+                        pulong->minimum, pulong->maximum, g_value_get_ulong (&value));
+        break;
       }
-      case G_TYPE_LONG: 
+      case G_TYPE_LONG:
       {
-	GParamSpecLong *plong = G_PARAM_SPEC_LONG (param);
-	g_print("%-23.23s Long. ", ""); 
-	if (readable) g_print("Range: %ld - %ld (Default %ld)",  
-			plong->minimum, plong->maximum, g_value_get_long (&value));
-	break;
+        GParamSpecLong *plong = G_PARAM_SPEC_LONG (param);
+        g_print("%-23.23s Long. ", "");
+        if (readable) g_print("Range: %ld - %ld (Default %ld)",
+                        plong->minimum, plong->maximum, g_value_get_long (&value));
+        break;
       }
-      case G_TYPE_UINT: 
+      case G_TYPE_UINT:
       {
-	GParamSpecUInt *puint = G_PARAM_SPEC_UINT (param);
-	g_print("%-23.23s Unsigned Integer. ", "");
-	if (readable) g_print("Range: %u - %u (Default %u)",  
-			puint->minimum, puint->maximum, g_value_get_uint (&value));
-	break;
+        GParamSpecUInt *puint = G_PARAM_SPEC_UINT (param);
+        g_print("%-23.23s Unsigned Integer. ", "");
+        if (readable) g_print("Range: %u - %u (Default %u)",
+                        puint->minimum, puint->maximum, g_value_get_uint (&value));
+        break;
       }
-      case G_TYPE_INT: 
+      case G_TYPE_INT:
       {
-	GParamSpecInt *pint = G_PARAM_SPEC_INT (param);
-	g_print("%-23.23s Integer. ", ""); 
-	if (readable) g_print("Range: %d - %d (Default %d)", 
-			pint->minimum, pint->maximum, g_value_get_int (&value));
-	break;
+        GParamSpecInt *pint = G_PARAM_SPEC_INT (param);
+        g_print("%-23.23s Integer. ", "");
+        if (readable) g_print("Range: %d - %d (Default %d)",
+                        pint->minimum, pint->maximum, g_value_get_int (&value));
+        break;
       }
-      case G_TYPE_UINT64: 
+      case G_TYPE_UINT64:
       {
-	GParamSpecUInt64 *puint64 = G_PARAM_SPEC_UINT64 (param);
-	g_print("%-23.23s Unsigned Integer64. ", ""); 
-	if (readable) g_print("Range: %" G_GUINT64_FORMAT " - %" G_GUINT64_FORMAT " (Default %" G_GUINT64_FORMAT ")", 
-			puint64->minimum, puint64->maximum, g_value_get_uint64 (&value));
-	break;
+        GParamSpecUInt64 *puint64 = G_PARAM_SPEC_UINT64 (param);
+        g_print("%-23.23s Unsigned Integer64. ", "");
+        if (readable) g_print("Range: %" G_GUINT64_FORMAT " - %"
+                        G_GUINT64_FORMAT " (Default %" G_GUINT64_FORMAT ")",
+                        puint64->minimum, puint64->maximum,
+                        g_value_get_uint64 (&value));
+        break;
       }
-      case G_TYPE_INT64: 
+      case G_TYPE_INT64:
       {
-	GParamSpecInt64 *pint64 = G_PARAM_SPEC_INT64 (param);
-	g_print("%-23.23s Integer64. ", ""); 
-	if (readable) g_print("Range: %" G_GINT64_FORMAT " - %" G_GINT64_FORMAT " (Default %" G_GINT64_FORMAT ")", 
-			pint64->minimum, pint64->maximum, g_value_get_int64 (&value));
-	break;
+        GParamSpecInt64 *pint64 = G_PARAM_SPEC_INT64 (param);
+        g_print("%-23.23s Integer64. ", "");
+        if (readable) g_print("Range: %" G_GINT64_FORMAT " - %" G_GINT64_FORMAT                         " (Default %" G_GINT64_FORMAT ")", pint64->minimum,
+                        pint64->maximum, g_value_get_int64 (&value));
+        break;
       }
-      case G_TYPE_FLOAT: 
+      case G_TYPE_FLOAT:
       {
-	GParamSpecFloat *pfloat = G_PARAM_SPEC_FLOAT (param);
-	g_print("%-23.23s Float. Default: %-8.8s %15.7g\n", "", "", 
-		g_value_get_float (&value));
-	g_print("%-23.23s Range: %15.7g - %15.7g", "", 
-	       pfloat->minimum, pfloat->maximum);
-	break;
+        GParamSpecFloat *pfloat = G_PARAM_SPEC_FLOAT (param);
+        g_print("%-23.23s Float. Default: %-8.8s %15.7g\n", "", "",
+                g_value_get_float (&value));
+        g_print("%-23.23s Range: %15.7g - %15.7g", "",
+               pfloat->minimum, pfloat->maximum);
+        break;
       }
-      case G_TYPE_DOUBLE: 
+      case G_TYPE_DOUBLE:
       {
-	GParamSpecDouble *pdouble = G_PARAM_SPEC_DOUBLE (param);
-	g_print("%-23.23s Double. Default: %-8.8s %15.7g\n", "", "", 
-		g_value_get_double (&value));
-	g_print("%-23.23s Range: %15.7g - %15.7g", "", 
-	       pdouble->minimum, pdouble->maximum);
-	break;
+        GParamSpecDouble *pdouble = G_PARAM_SPEC_DOUBLE (param);
+        g_print("%-23.23s Double. Default: %-8.8s %15.7g\n", "", "",
+                g_value_get_double (&value));
+        g_print("%-23.23s Range: %15.7g - %15.7g", "",
+               pdouble->minimum, pdouble->maximum);
+        break;
       }
       default:
-        if (param->value_type == GST_TYPE_FILENAME) {
-          g_print("%-23.23s Filename", "");
-	}
+        if (param->value_type == GST_TYPE_URI) {
+          g_print("%-23.23s URI", "");
+        }
         if (param->value_type == GST_TYPE_CAPS) {
           GstCaps *caps = g_value_peek_pointer (&value);
 
-	  if (!caps) 
+          if (!caps)
             g_print("%-23.23s Caps (NULL)", "");
-	  else {
+          else {
             print_caps (caps, "                           ");
-	  }
-	}
+          }
+        }
         else if (G_IS_PARAM_SPEC_ENUM (param)) {
           GEnumValue *values;
-	  guint j = 0;
-	  gint enum_value;
+          guint j = 0;
+          gint enum_value;
 
-	  values = G_ENUM_CLASS (g_type_class_ref (param->value_type))->values;
-	  enum_value = g_value_get_enum (&value);
+          values = G_ENUM_CLASS (g_type_class_ref (param->value_type))->values;
+          enum_value = g_value_get_enum (&value);
 
-	  while (values[j].value_name) {
-	    if (values[j].value == enum_value)
-	      break;
-	    j++; 
-	  }
+          while (values[j].value_name) {
+            if (values[j].value == enum_value)
+              break;
+            j++; 
+          }
 
-          g_print ("%-23.23s Enum \"%s\" (default %d, \"%s\")", "", 
-			  g_type_name (G_VALUE_TYPE (&value)),
-			  enum_value, values[j].value_nick);
+          g_print ("%-23.23s Enum \"%s\" (default %d, \"%s\")", "",
+                        g_type_name (G_VALUE_TYPE (&value)),
+                        enum_value, values[j].value_nick);
 
-	  j = 0;
-	  while (values[j].value_name) {
-            g_print("\n%-23.23s    (%d): \t%s", "", 
-			    values[j].value, values[j].value_nick);
-	    j++; 
-	  }
-	  /* g_type_class_unref (ec); */
-	}
+          j = 0;
+          while (values[j].value_name) {
+            g_print("\n%-23.23s    (%d): \t%s", "",
+                        values[j].value, values[j].value_nick);
+            j++;
+          }
+          /* g_type_class_unref (ec); */
+        }
         else if (G_IS_PARAM_SPEC_FLAGS (param)) {
           GFlagsValue *values;
-	  guint j = 0;
-	  gint flags_value;
-	  GString *flags = NULL;
+          guint j = 0;
+          gint flags_value;
+          GString *flags = NULL;
 
-	  values = G_FLAGS_CLASS (g_type_class_ref (param->value_type))->values;
-	  flags_value = g_value_get_flags (&value);
+          values = G_FLAGS_CLASS (g_type_class_ref (param->value_type))->values;
+          flags_value = g_value_get_flags (&value);
 
-	  while (values[j].value_name) {
-	    if (values[j].value & flags_value) {
-	      if (flags) {
-	        g_string_append_printf (flags, " | %s", values[j].value_nick);
-	      }
-	      else {
-	        flags = g_string_new (values[j].value_nick);
-	      }
-	    }
-	    j++;
-	  }
+          while (values[j].value_name) {
+            if (values[j].value & flags_value) {
+              if (flags) {
+                g_string_append_printf (flags, " | %s", values[j].value_nick);
+              }
+              else {
+                flags = g_string_new (values[j].value_nick);
+              }
+            }
+            j++;
+          }
 
-          g_print ("%-23.23s Flags \"%s\" (default %d, \"%s\")", "", 
-			  g_type_name (G_VALUE_TYPE (&value)),
-			  flags_value, (flags ? flags->str : "(none)"));
+          g_print("%-23.23s Flags \"%s\" (default %d, \"%s\")", "",
+                      g_type_name (G_VALUE_TYPE (&value)),
+                      flags_value, (flags ? flags->str : "(none)"));
 
-	  j = 0;
-	  while (values[j].value_name) {
-            g_print("\n%-23.23s    (%d): \t%s", "", 
-			    values[j].value, values[j].value_nick);
-	    j++; 
-	  }
+          j = 0;
+          while (values[j].value_name) {
+            g_print("\n%-23.23s    (%d): \t%s", "",
+                      values[j].value, values[j].value_nick);
+            j++;
+          }
 
-	  if (flags)
-	    g_string_free (flags, TRUE);
-	}
-	else if (G_IS_PARAM_SPEC_OBJECT (param)) {
-	  g_print("%-23.23s Object of type \"%s\"", "",
-			  g_type_name(param->value_type));
+          if (flags)
+            g_string_free (flags, TRUE);
+        }
+        else if (G_IS_PARAM_SPEC_OBJECT (param)) {
+          g_print("%-23.23s Object of type \"%s\"", "",
+                      g_type_name(param->value_type));
+        }
+        else {
+          g_print ("%-23.23s Unknown type %ld \"%s\"", "", param->value_type,
+                      g_type_name(param->value_type));
         }
-	else {
-          g_print ("%-23.23s Unknown type %ld \"%s\"", "",param->value_type, 
-		  	g_type_name(param->value_type));
-	}
         break;
     }
-    if (!readable) 
+    if (!readable)
       g_print (" Write only\n");
-    else 
+    else
       g_print ("\n");
   }
-  if (num_properties == 0) 
+  if (num_properties == 0)
     g_print ("  none\n");
 }
 
+static char *
+get_rank_name (gint rank)
+{
+  switch(rank){
+    case GST_RANK_NONE:
+      return "none";
+    case GST_RANK_MARGINAL:
+      return "marginal";
+    case GST_RANK_SECONDARY:
+      return "secondary";
+    case GST_RANK_PRIMARY:
+      return "primary";
+    default:
+      return "unknown";
+  }
+}
+
 static gint
 print_element_info (GstElementFactory *factory)
 {
@@ -459,13 +483,11 @@ print_element_info (GstElementFactory *factory)
   gstelement_class = GST_ELEMENT_CLASS (G_OBJECT_GET_CLASS (element));
 
   g_print ("Factory Details:\n");
-  g_print ("  Long name:\t%s\n",   factory->details->longname);
-  g_print ("  Class:\t%s\n",       factory->details->klass);
-  g_print ("  License:\t%s\n",     factory->details->license);
-  g_print ("  Description:\t%s\n", factory->details->description);
-  g_print ("  Version:\t%s\n",     factory->details->version);
-  g_print ("  Author(s):\t%s\n",   factory->details->author);
-  g_print ("  Copyright:\t%s\n",   factory->details->copyright);
+  g_print ("  Long name:\t%s\n",   factory->details.longname);
+  g_print ("  Class:\t%s\n",       factory->details.klass);
+  g_print ("  Description:\t%s\n", factory->details.description);
+  g_print ("  Author(s):\t%s\n",   factory->details.author);
+  g_print ("  Rank:\t\t%s\n",      get_rank_name(GST_PLUGIN_FEATURE(factory)->rank));
   g_print ("\n");
 
   output_hierarchy (G_OBJECT_TYPE (element), 0, &maxlevel);
@@ -498,7 +520,7 @@ print_element_info (GstElementFactory *factory)
 
       if (padtemplate->caps) {
         g_print ("    Capabilities:\n");
-	print_caps (padtemplate->caps, "      ");
+        print_caps (padtemplate->caps, "      ");
       }
 
       g_print ("\n");
@@ -552,7 +574,7 @@ print_element_info (GstElementFactory *factory)
 
   if (element->loopfunc)
     g_print ("  loopfunc()-based element: %s\n",
-	    GST_DEBUG_FUNCPTR_NAME (element->loopfunc));
+            GST_DEBUG_FUNCPTR_NAME (element->loopfunc));
   else
     g_print ("  No loopfunc(), must be chain-based or not configured yet\n");
 
@@ -586,6 +608,7 @@ print_element_info (GstElementFactory *factory)
     g_print ("  none\n");
   }
 
+#ifndef GST_DISABLE_INDEX
   g_print ("\nIndexing capabilities:\n");
   if (gst_element_is_indexable (element)) {
     g_print ("  element can do indexing\n");
@@ -593,6 +616,7 @@ print_element_info (GstElementFactory *factory)
   else {
     g_print ("  none\n");
   }
+#endif
 
   g_print ("\nPads:\n");
   if (element->numpads) {
@@ -618,47 +642,47 @@ print_element_info (GstElementFactory *factory)
       g_print ("    Implementation:\n");
       if (realpad->chainfunc)
         g_print ("      Has chainfunc(): %s\n",
-	        GST_DEBUG_FUNCPTR_NAME (realpad->chainfunc));
+                GST_DEBUG_FUNCPTR_NAME (realpad->chainfunc));
       if (realpad->getfunc)
         g_print ("      Has getfunc(): %s\n",
-	        GST_DEBUG_FUNCPTR_NAME (realpad->getfunc));
+                GST_DEBUG_FUNCPTR_NAME (realpad->getfunc));
       if (realpad->formatsfunc != gst_pad_get_formats_default) {
         g_print ("      Supports seeking/conversion/query formats:\n");
-	print_formats (gst_pad_get_formats (GST_PAD (realpad)));
+        print_formats (gst_pad_get_formats (GST_PAD (realpad)));
       }
       if (realpad->convertfunc != gst_pad_convert_default)
         g_print ("      Has custom convertfunc(): %s\n",
-	        GST_DEBUG_FUNCPTR_NAME (realpad->convertfunc));
+                GST_DEBUG_FUNCPTR_NAME (realpad->convertfunc));
       if (realpad->eventfunc != gst_pad_event_default)
         g_print ("      Has custom eventfunc(): %s\n",
-	        GST_DEBUG_FUNCPTR_NAME (realpad->eventfunc));
+                GST_DEBUG_FUNCPTR_NAME (realpad->eventfunc));
       if (realpad->eventmaskfunc != gst_pad_get_event_masks_default) {
         g_print ("        Provides event masks:\n");
-	print_event_masks (gst_pad_get_event_masks (GST_PAD (realpad)));
+        print_event_masks (gst_pad_get_event_masks (GST_PAD (realpad)));
       }
       if (realpad->queryfunc != gst_pad_query_default)
         g_print ("      Has custom queryfunc(): %s\n",
-	        GST_DEBUG_FUNCPTR_NAME (realpad->queryfunc));
+                GST_DEBUG_FUNCPTR_NAME (realpad->queryfunc));
       if (realpad->querytypefunc != gst_pad_get_query_types_default) {
         g_print ("        Provides query types:\n");
-	print_query_types (gst_pad_get_query_types (GST_PAD (realpad)));
+        print_query_types (gst_pad_get_query_types (GST_PAD (realpad)));
       }
 
       if (realpad->intlinkfunc != gst_pad_get_internal_links_default)
         g_print ("      Has custom intconnfunc(): %s\n",
-	        GST_DEBUG_FUNCPTR_NAME(realpad->intlinkfunc));
+                GST_DEBUG_FUNCPTR_NAME(realpad->intlinkfunc));
 
       if (realpad->bufferpoolfunc)
         g_print ("      Has bufferpoolfunc(): %s\n",
-	        GST_DEBUG_FUNCPTR_NAME(realpad->bufferpoolfunc));
+                GST_DEBUG_FUNCPTR_NAME(realpad->bufferpoolfunc));
 
       if (pad->padtemplate)
         g_print ("    Pad Template: '%s'\n",
-	        pad->padtemplate->name_template);
+                pad->padtemplate->name_template);
 
       if (realpad->caps) {
         g_print ("    Capabilities:\n");
-	print_caps (realpad->caps, "      ");
+        print_caps (realpad->caps, "      ");
       }
     }
   } else
@@ -671,7 +695,7 @@ print_element_info (GstElementFactory *factory)
     GstDParamManager* dpman;
     GParamSpec** specs;
     gint x;
-    
+
     g_print ("\nDynamic Parameters:\n");
     if((dpman = gst_dpman_get_manager (element))) {
       specs = gst_dpman_list_dparam_specs (dpman);
@@ -679,23 +703,24 @@ print_element_info (GstElementFactory *factory)
         g_print ("  %-20.20s: ", g_param_spec_get_name (specs[x]));
 
         switch (G_PARAM_SPEC_VALUE_TYPE (specs[x])) {
-          case G_TYPE_INT64: 
-            g_print ("64 Bit Integer (Default %" G_GINT64_FORMAT ", Range %" G_GINT64_FORMAT " -> %" G_GINT64_FORMAT ")", 
+          case G_TYPE_INT64:
+            g_print ("64 Bit Integer (Default %" G_GINT64_FORMAT ", Range %"
+                         G_GINT64_FORMAT " -> %" G_GINT64_FORMAT ")",
             ((GParamSpecInt64 *) specs[x])->default_value,
-            ((GParamSpecInt64 *) specs[x])->minimum, 
+            ((GParamSpecInt64 *) specs[x])->minimum,
             ((GParamSpecInt64 *) specs[x])->maximum);
             break;
-          case G_TYPE_INT: 
-            g_print ("Integer (Default %d, Range %d -> %d)", 
+          case G_TYPE_INT:
+            g_print ("Integer (Default %d, Range %d -> %d)",
             ((GParamSpecInt *) specs[x])->default_value,
-            ((GParamSpecInt *) specs[x])->minimum, 
+            ((GParamSpecInt *) specs[x])->minimum,
             ((GParamSpecInt *) specs[x])->maximum);
             break;
-          case G_TYPE_FLOAT: 
-	    g_print ("Float. Default: %-8.8s %15.7g\n", "",
+          case G_TYPE_FLOAT:
+            g_print ("Float. Default: %-8.8s %15.7g\n", "",
               ((GParamSpecFloat *) specs[x])->default_value);
-	    g_print ("%-23.23s Range: %15.7g - %15.7g", "", 
-              ((GParamSpecFloat *) specs[x])->minimum, 
+            g_print ("%-23.23s Range: %15.7g - %15.7g", "",
+              ((GParamSpecFloat *) specs[x])->minimum,
               ((GParamSpecFloat *) specs[x])->maximum);
             break;
         default: g_print ("unknown %ld", G_PARAM_SPEC_VALUE_TYPE (specs[x]));
@@ -709,14 +734,14 @@ print_element_info (GstElementFactory *factory)
     }
   }
 
-  /* Signals/Actions Block */  
+  /* Signals/Actions Block */
   {
     guint *signals;
     guint nsignals;
     gint i, k;
     GSignalQuery *query;
-    
-    signals = g_signal_list_ids (G_OBJECT_TYPE (element), &nsignals);
+    GType type;
+
     for (k = 0; k < 2; k++) {
       gint counted = 0;
 
@@ -725,48 +750,51 @@ print_element_info (GstElementFactory *factory)
       else
         g_print ("\nElement Actions:\n");
 
-      for (i = 0; i < nsignals; i++) {
-        gint n_params;
-        GType return_type;
-        const GType *param_types;
-        gint j;
-      
-        query = g_new0 (GSignalQuery,1);
-        g_signal_query (signals[i], query);
+      for(type = G_OBJECT_TYPE(element); type; type = g_type_parent(type)){
+        signals = g_signal_list_ids (type, &nsignals);
 
-	if ((k == 0 && !(query->signal_flags & G_SIGNAL_ACTION)) ||
-	    (k == 1 &&  (query->signal_flags & G_SIGNAL_ACTION))) {
-          n_params = query->n_params;
-          return_type = query->return_type;
-          param_types = query->param_types;
+	for (i = 0; i < nsignals; i++) {
+	  gint n_params;
+	  GType return_type;
+	  const GType *param_types;
+	  gint j;
 
-          g_print ("  \"%s\" :\t %s user_function (%s* object", 
-	          query->signal_name, g_type_name (return_type),
-		  g_type_name (G_OBJECT_TYPE (element)));
+	  query = g_new0 (GSignalQuery,1);
+	  g_signal_query (signals[i], query);
 
-          for (j = 0; j < n_params; j++) {
-            g_print (",\n    \t\t\t\t%s arg%d", g_type_name (param_types[j]), j);
-          }
-	  if (k == 0)
-            g_print (",\n    \t\t\t\tgpointer user_data);\n");
-	  else
-            g_print (");\n");
+	  if ((k == 0 && !(query->signal_flags & G_SIGNAL_ACTION)) ||
+	      (k == 1 &&  (query->signal_flags & G_SIGNAL_ACTION))) {
+	    n_params = query->n_params;
+	    return_type = query->return_type;
+	    param_types = query->param_types;
 
-	  counted++;
+	    g_print ("  \"%s\" :\t %s user_function (%s* object",
+		    query->signal_name, g_type_name (return_type),
+		    g_type_name (type));
+
+	    for (j = 0; j < n_params; j++) {
+	      g_print (",\n    \t\t\t\t%s arg%d", g_type_name(param_types[j]), j);
+	    }
+	    if (k == 0)
+	      g_print (",\n    \t\t\t\tgpointer user_data);\n");
+	    else
+	      g_print (");\n");
+
+	    counted++;
+	  }
+
+	  g_free (query);
 	}
-
-        g_free (query);
       }
       if (counted == 0) g_print ("  none\n");
     }
   }
-  
 
   /* for compound elements */
   if (GST_IS_BIN (element)) {
     g_print ("\nChildren:\n");
     children = (GList *) gst_bin_get_list (GST_BIN (element));
-    if (!children) 
+    if (!children)
       g_print ("  none\n");
     else {
       while (children) {
@@ -781,8 +809,8 @@ print_element_info (GstElementFactory *factory)
   return 0;
 }
 
-static void 
-print_element_list (void) 
+static void
+print_element_list (void)
 {
   GList *plugins;
 
@@ -790,7 +818,7 @@ print_element_list (void)
   while (plugins) {
     GList *features;
     GstPlugin *plugin;
-    
+
     plugin = (GstPlugin*)(plugins->data);
     plugins = g_list_next (plugins);
 
@@ -804,53 +832,62 @@ print_element_list (void)
         GstElementFactory *factory;
 
         factory = GST_ELEMENT_FACTORY (feature);
-        g_print ("%s:  %s: %s\n", plugin->name, 
-	        GST_PLUGIN_FEATURE_NAME (factory) ,factory->details->longname);
+        g_print ("%s:  %s: %s\n", plugin->desc.name,
+                GST_PLUGIN_FEATURE_NAME (factory) ,factory->details.longname);
       }
+#ifndef GST_DISABLE_AUTOPLUG
       else if (GST_IS_AUTOPLUG_FACTORY (feature)) {
         GstAutoplugFactory *factory;
 
         factory = GST_AUTOPLUG_FACTORY (feature);
-        g_print ("%s:  %s: %s\n", plugin->name, 
-	        GST_PLUGIN_FEATURE_NAME (factory), factory->longdesc);
+        g_print ("%s:  %s: %s\n", plugin->desc.name,
+                GST_PLUGIN_FEATURE_NAME (factory), factory->longdesc);
       }
+#endif
+#ifndef GST_DISABLE_INDEX
       else if (GST_IS_INDEX_FACTORY (feature)) {
         GstIndexFactory *factory;
 
         factory = GST_INDEX_FACTORY (feature);
-        g_print ("%s:  %s: %s\n", plugin->name, 
-	        GST_PLUGIN_FEATURE_NAME (factory), factory->longdesc);
+        g_print ("%s:  %s: %s\n", plugin->desc.name,
+                GST_PLUGIN_FEATURE_NAME (factory), factory->longdesc);
       }
-      else if (GST_IS_TYPE_FACTORY (feature)) {
-        GstTypeFactory *factory;
+#endif
+      else if (GST_IS_TYPE_FIND_FACTORY (feature)) {
+        GstTypeFindFactory *factory;
 
-        factory = GST_TYPE_FACTORY (feature);
-        g_print ("%s type:  %s: %s\n", plugin->name, 
-	        factory->mime, factory->exts);
-
-        if (factory->typefindfunc)
-          g_print ("      Has typefind function: %s\n",
-	          GST_DEBUG_FUNCPTR_NAME (factory->typefindfunc));
+        factory = GST_TYPE_FIND_FACTORY (feature);
+        if (factory->extensions) {
+	  guint i = 0;
+          g_print ("%s type: ", plugin->desc.name);
+	  while (factory->extensions[i]) {
+	    g_print ("%s%s", i > 0 ? ", " : "", factory->extensions[i]);
+	    i++;
+	  }
+	} else
+          g_print ("%s type: N/A\n", plugin->desc.name);
       }
       else if (GST_IS_SCHEDULER_FACTORY (feature)) {
         GstSchedulerFactory *factory;
 
         factory = GST_SCHEDULER_FACTORY (feature);
-        g_print ("%s:  %s: %s\n", plugin->name, 
-	        GST_PLUGIN_FEATURE_NAME (factory), factory->longdesc);
+        g_print ("%s:  %s: %s\n", plugin->desc.name,
+                GST_PLUGIN_FEATURE_NAME (factory), factory->longdesc);
       }
+#ifndef GST_DISABLE_URI
       else if (GST_IS_URI_HANDLER (feature)) {
         GstURIHandler *handler;
 
         handler = GST_URI_HANDLER (feature);
-        g_print ("%s:  %s: \"%s\" (%s) element \"%s\" property \"%s\"\n", plugin->name, 
-	        GST_PLUGIN_FEATURE_NAME (handler), handler->uri, handler->longdesc,
-		handler->element, handler->property);
+        g_print ("%s:  %s: \"%s\" (%s) element \"%s\" property \"%s\"\n",
+                plugin->desc.name, GST_PLUGIN_FEATURE_NAME (handler), handler->uri,
+                handler->longdesc, handler->element, handler->property);
       }
+#endif
       else {
-        g_print ("%s:  %s (%s)\n", plugin->name, 
-	        GST_PLUGIN_FEATURE_NAME (feature), 
-	  	g_type_name (G_OBJECT_TYPE (feature)));
+        g_print ("%s:  %s (%s)\n", plugin->desc.name,
+                GST_PLUGIN_FEATURE_NAME (feature),
+                g_type_name (G_OBJECT_TYPE (feature)));
       }
 
       features = g_list_next (features);
@@ -869,11 +906,16 @@ print_plugin_info (GstPlugin *plugin)
   gint num_schedulers = 0;
   gint num_indexes = 0;
   gint num_other = 0;
-  
+
   g_print ("Plugin Details:\n");
-  g_print ("  Name:\t\t%s\n",    plugin->name);
-  g_print ("  Long Name:\t%s\n", plugin->longname);
-  g_print ("  Filename:\t%s\n",  plugin->filename);
+  g_print ("  Name:\t\t%s\n",	    plugin->desc.name);
+  g_print ("  Description:\t%s\n",  plugin->desc.description);
+  g_print ("  Filename:\t%s\n",	    plugin->filename);
+  g_print ("  Version:\t%s\n",	    plugin->desc.version);
+  g_print ("  License:\t%s\n",	    plugin->desc.license);
+  g_print ("  Copyright:\t%s\n",    plugin->desc.copyright);
+  g_print ("  Package:\t%s\n",	    plugin->desc.package);
+  g_print ("  Origin URL:\t%s\n",   plugin->desc.origin);
   g_print ("\n");
 
   features = gst_plugin_get_feature_list (plugin);
@@ -888,9 +930,10 @@ print_plugin_info (GstPlugin *plugin)
 
       factory = GST_ELEMENT_FACTORY (feature);
       g_print ("  %s: %s\n", GST_OBJECT_NAME (factory),
-	      factory->details->longname);
+              factory->details.longname);
       num_elements++;
     }
+#ifndef GST_DISABLE_AUTOPLUG
     else if (GST_IS_AUTOPLUG_FACTORY (feature)) {
       GstAutoplugFactory *factory;
 
@@ -898,6 +941,8 @@ print_plugin_info (GstPlugin *plugin)
       g_print ("  %s: %s\n", GST_OBJECT_NAME (factory), factory->longdesc);
       num_autoplug++;
     }
+#endif
+#ifndef GST_DISABLE_INDEX
     else if (GST_IS_INDEX_FACTORY (feature)) {
       GstIndexFactory *factory;
 
@@ -905,15 +950,21 @@ print_plugin_info (GstPlugin *plugin)
       g_print ("  %s: %s\n", GST_OBJECT_NAME (factory), factory->longdesc);
       num_indexes++;
     }
-    else if (GST_IS_TYPE_FACTORY (feature)) {
-      GstTypeFactory *factory;
+#endif
+    else if (GST_IS_TYPE_FIND_FACTORY (feature)) {
+      GstTypeFindFactory *factory;
 
-      factory = GST_TYPE_FACTORY (feature);
-      g_print ("  %s: %s\n", factory->mime, factory->exts);
+      factory = GST_TYPE_FIND_FACTORY (feature);
+      if (factory->extensions) {
+	guint i = 0;
+	g_print ("%s type: ", plugin->desc.name);
+	while (factory->extensions[i]) {
+	  g_print ("%s%s", i > 0 ? ", " : "", factory->extensions[i]);
+	  i++;
+	}
+      } else
+	g_print ("%s type: N/A\n", plugin->desc.name);
 
-      if (factory->typefindfunc)
-        g_print ("      Has typefind function: %s\n", 
-	        GST_DEBUG_FUNCPTR_NAME (factory->typefindfunc));
       num_types++;
     }
     else if (GST_IS_SCHEDULER_FACTORY (feature)) {
@@ -924,8 +975,8 @@ print_plugin_info (GstPlugin *plugin)
       num_schedulers++;
     }
     else {
-      g_print ("  %s (%s)\n", gst_object_get_name (GST_OBJECT (feature)), 
-		             g_type_name (G_OBJECT_TYPE (feature)));
+      g_print ("  %s (%s)\n", gst_object_get_name (GST_OBJECT (feature)),
+                             g_type_name (G_OBJECT_TYPE (feature)));
       num_other++;
     }
     num_features++;
@@ -944,28 +995,27 @@ print_plugin_info (GstPlugin *plugin)
     g_print ("  +-- %d indexes\n", num_indexes);
   if (num_other > 0)
     g_print ("  +-- %d other objects\n", num_other);
-  
+
   g_print ("\n");
 }
 
-
-int 
-main (int argc, char *argv[]) 
+int
+main (int argc, char *argv[])
 {
   GstElementFactory *factory;
   GstPlugin *plugin;
   gchar *so;
   struct poptOption options[] = {
-    {"gst-inspect-plugin",  'p',  POPT_ARG_STRING|POPT_ARGFLAG_STRIP,   NULL,   0,
-	           "Show plugin details", NULL},
-    {"gst-inspect-scheduler",  's',  POPT_ARG_STRING|POPT_ARGFLAG_STRIP,   NULL,   0,
-	           "Show scheduler details", NULL},
+    {"gst-inspect-plugin", 'p', POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, 0,
+                   "Show plugin details", NULL},
+    {"gst-inspect-scheduler", 's', POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, 0,
+                   "Show scheduler details", NULL},
     POPT_TABLEEND
   };
 
   gst_init_with_popt_table (&argc, &argv, options);
   gst_control_init (&argc, &argv);
-  
+
   /* if no arguments, print out list of elements */
   if (argc == 1) {
     print_element_list();
@@ -977,7 +1027,7 @@ main (int argc, char *argv[])
       g_print ("Usage: %s\t\t\tList all registered elements\n",argv[0]);
       g_print ("       %s element-name\tShow element details\n",argv[0]);
       g_print ("       %s plugin-name[.so]\tShow information about plugin\n",
-	      argv[0]);
+              argv[0]);
       return 0;
     }
 
@@ -989,34 +1039,45 @@ main (int argc, char *argv[])
       if (factory)
         return print_element_info (factory);
       else {
-	 GstPluginFeature* feature;
+         GstPluginFeature* feature;
 
-	 /* FIXME implement other pretty print function for these */
-	 feature = gst_registry_pool_find_feature (argv[1], GST_TYPE_SCHEDULER_FACTORY);
-	 if (feature) {
+         /* FIXME implement other pretty print function for these */
+         feature = gst_registry_pool_find_feature (argv[1],
+                                                   GST_TYPE_SCHEDULER_FACTORY);
+         if (feature) {
            g_print ("%s: a scheduler\n", argv[1]);
-	   return 0;
-	 }
-	 feature = gst_registry_pool_find_feature (argv[1], GST_TYPE_INDEX_FACTORY);
-	 if (feature) {
+           return 0;
+         }
+#ifndef GST_DISABLE_INDEX
+         feature = gst_registry_pool_find_feature (argv[1],
+                                                   GST_TYPE_INDEX_FACTORY);
+         if (feature) {
            g_print ("%s: an index\n", argv[1]);
-	   return 0;
-	 }
-	 feature = gst_registry_pool_find_feature (argv[1], GST_TYPE_AUTOPLUG_FACTORY);
-	 if (feature) {
+           return 0;
+         }
+#endif
+#ifndef GST_DISABLE_AUTOPLUG
+         feature = gst_registry_pool_find_feature (argv[1],
+                                                   GST_TYPE_AUTOPLUG_FACTORY);
+         if (feature) {
            g_print ("%s: an autoplugger\n", argv[1]);
-	   return 0;
-	 }
-	 feature = gst_registry_pool_find_feature (argv[1], GST_TYPE_TYPE_FACTORY);
-	 if (feature) {
-           g_print ("%s: an type\n", argv[1]);
-	   return 0;
-	 }
-	 feature = gst_registry_pool_find_feature (argv[1], GST_TYPE_URI_HANDLER);
-	 if (feature) {
+           return 0;
+         }
+#endif
+         feature = gst_registry_pool_find_feature (argv[1],
+                                                   GST_TYPE_TYPE_FIND_FACTORY);
+         if (feature) {
+           g_print ("%s: a typefind function\n", argv[1]);
+           return 0;
+         }
+#ifndef GST_DISABLE_URI
+         feature = gst_registry_pool_find_feature (argv[1],
+                                                   GST_TYPE_URI_HANDLER);
+         if (feature) {
            g_print ("%s: an uri handler\n", argv[1]);
-	   return 0;
-	 }
+           return 0;
+         }
+#endif
       }
     } else {
       /* strip the .so */
